"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("@tomei/general");
const account_system_entity_1 = require("../account-system-entity/account-system-entity");
const account_repository_1 = require("./account.repository");
const getConfig = require('../config');
class Account extends account_system_entity_1.AccountSystemEntity {
    get IsNewRecord() {
        return this._IsNewRecord;
    }
    set IsNewRecord(record) {
        this._IsNewRecord = record;
    }
    get RepositoryBase() {
        return Account._RepositoryBase;
    }
    get ObjectId() {
        return this.AccountNo;
    }
    get ObjectName() {
        return this.Name;
    }
    get TableName() {
        return 'finance_Account';
    }
    constructor(dbTransaction, accountNo) {
        super();
        this.AccountNo = 'New';
        this.ParentAccountNo = '';
        this.Name = '';
        this.Description = '';
        this.AccountType = '';
        this.AccountSubtype = '';
        this.OwnerId = '';
        this.OwnerType = '';
        this.RelatedObjectId = '';
        this.RelatedObjectType = '';
        this.CreatedById = '';
        this.UpdatedById = '';
        this._IsNewRecord = true;
        if (dbTransaction) {
            this._DbTransaction = dbTransaction;
        }
        if (accountNo) {
            this.RepositoryBase.findOne({
                where: {
                    AccountNo: accountNo,
                },
                transaction: dbTransaction,
            })
                .then((accountData) => {
                if (accountData) {
                    this.IsNewRecord = false;
                    this.CompanyId = accountData.CompanyId;
                    this.ParentAccountNo = accountData.ParentAccountNo;
                    this.Name = accountData.Name;
                    this.Description = accountData.Description;
                    this.AccountType = accountData.AccountType;
                    this.AccountSubtype = accountData.AccountSubType;
                    this.OwnerId = accountData.OwnerId;
                    this.OwnerType = accountData.OwnerType;
                    this.RelatedObjectId = accountData.RelatedObjectId;
                    this.RelatedObjectType = accountData.RelatedObjectType;
                    this.CreatedById = accountData.CreatedById;
                    this.CreatedAt = accountData.CreatedAt;
                    this.UpdatedById = accountData.UpdatedById;
                    this.UpdatedAt = accountData.UpdatedAt;
                    this.AccSystemRefId = accountData.AccSystemRefId;
                    this.PostedToAccSystemYN = accountData.PostedToAccSystemYN;
                    this.PostedById = accountData.PostedById;
                    this.PostedDateTime = accountData.PostedDateTime;
                }
                else {
                    const notFoundError = new general_1.RecordNotFoundError('No Record Found.');
                    throw notFoundError;
                }
            })
                .catch((err) => {
                console.log('Account Class constructor err: ', err);
            });
        }
    }
    init(accountData) {
        this.IsNewRecord = false;
        this.CompanyId = accountData.CompanyId;
        this.ParentAccountNo = accountData.ParentAccountNo;
        this.Name = accountData.Name;
        this.Description = accountData.Description;
        this.AccountType = accountData.AccountType;
        this.AccountSubtype = accountData.AccountSubType;
        this.OwnerId = accountData.OwnerId;
        this.OwnerType = accountData.OwnerType;
        this.RelatedObjectId = accountData.RelatedObjectId;
        this.RelatedObjectType = accountData.RelatedObjectType;
        this.CreatedById = accountData.CreatedById;
        this.CreatedAt = accountData.CreatedAt;
        this.UpdatedById = accountData.UpdatedById;
        this.UpdatedAt = accountData.UpdatedAt;
        this.AccSystemRefId = accountData.AccSystemRefId;
        this.PostedToAccSystemYN = accountData.PostedToAccSystemYN;
        this.PostedById = accountData.PostedById;
        this.PostedDateTime = accountData.PostedDateTime;
    }
    static initAccount(dbTransaction, accountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const accountData = yield Account._RepositoryBase.findOne({
                    where: {
                        AccountNo: accountNo,
                    },
                    transaction: dbTransaction,
                });
                if (accountData) {
                    const account = new Account(dbTransaction);
                    account.init(accountData.get({ plain: true }));
                    return account;
                }
                else {
                    const notFoundError = new general_1.RecordNotFoundError('No Record Found.');
                    throw notFoundError;
                }
            }
            catch (error) {
                console.log('Account Class constructor err: ', error);
                throw error;
            }
        });
    }
    static initialize() {
        const dbTransaction = null;
        let defaultAccountReceivable;
        let defaultAccountPayable;
        try {
            defaultAccountReceivable = new Account(dbTransaction, 'EZC-AR');
        }
        catch (err) {
            defaultAccountReceivable = new Account(dbTransaction);
            defaultAccountReceivable.AccountNo = null;
            defaultAccountReceivable.AccountType = 'Account Receivable';
            defaultAccountReceivable.OwnerId = 'System';
            defaultAccountReceivable.OwnerType = '';
        }
        try {
            defaultAccountPayable = new Account(dbTransaction, 'EZC-AP');
        }
        catch (err) {
            defaultAccountPayable = new Account(dbTransaction);
            defaultAccountPayable.AccountNo = null;
            defaultAccountPayable.AccountType = 'Account Payable';
            defaultAccountPayable.OwnerId = 'System';
        }
    }
    validateAccountValue() {
        if (!this.AccSystemRefId) {
            throw new Error('AccSystemAccountId is required.');
        }
        if (!this.AccountType) {
            throw new Error('AccountType is required.');
        }
    }
    isParentAccountExists() {
        if (this.ParentAccountNo)
            return true;
        return false;
    }
    save(AccSystemRefId, userId, dbTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            this.AccSystemRefId = AccSystemRefId;
            const data = yield this.RepositoryBase.create({
                AccountNo: this.AccountNo,
                AccSystemRefId: this.AccSystemRefId,
                Name: this.Name,
                Description: this.Description,
                AccountType: this.AccountType,
                AccountSubtype: this.AccountSubtype,
                OwnerId: this.OwnerId,
                OwnerType: this.OwnerType,
                RelatedObjectId: this.RelatedObjectId,
                RelatedObjectType: this.RelatedObjectType,
                CreatedAt: new Date(),
                CreatedById: userId,
            }, dbTransaction);
            return data;
        });
    }
}
exports.default = Account;
Account._RepositoryBase = new account_repository_1.AccountRepository();
//# sourceMappingURL=account.js.map