import { IAddress, IRepositoryBase, IPerson, LoginUserBase } from '@tomei/general';
import FinanceCustomerModel from '../models/customer.entity';
import { AccountSystemEntity } from '../account-system-entity/account-system-entity';
import { FinanceCustomerRepository } from './finance-customer.repository';
import Account from '../account/account';
export default abstract class FinanceCustomerBase extends AccountSystemEntity implements IPerson {
    CustomerId: string;
    CustSystemCode: string;
    CustSystemRefId: string;
    protected abstract ParentARAccountNo: string;
    protected abstract ParentAPAccountNo: string;
    protected abstract ARAccountNo: string;
    protected abstract APAccountNo: string;
    abstract FullName: string;
    abstract IDNo: string;
    abstract IDType: string;
    abstract ContactNo: string;
    abstract Email: string;
    abstract DefaultAddress: IAddress;
    private _AccountReceivable;
    private _AccountPayable;
    private static _AccountRepository;
    getDetails(): Promise<{
        FullName: string;
        IDNo: string;
        IDType: string;
        Email: string;
        ContactNo: string;
    }>;
    protected static _FinanceCustomerRepository: FinanceCustomerRepository;
    RepositoryBase: IRepositoryBase<any>;
    constructor(custSystemRefId?: string, custSystemCode?: string, sFinanceCompanyId?: string);
    static initCustomer<C extends FinanceCustomerBase>(this: new () => C, custSystemRefId: string, custSystemCode: string, sFinanceCompanyId: string): Promise<C>;
    get AccountReceivable(): Promise<Account>;
    get AccountPayable(): Promise<Account>;
    abstract getBillingAddress(params: any): Promise<IAddress>;
    init(person: LoginUserBase): void;
    save(accSystemRefId: string, custSystemCode: string, custSystemRefId: string, dbTransaction?: any): Promise<FinanceCustomerModel>;
    static isFinanceCustomerExist(custSystemRefId: string, custSystemCode: string, sFinanceCompanyId: string): Promise<boolean>;
}
