"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("@tomei/general");
const account_system_entity_1 = require("../account-system-entity/account-system-entity");
const finance_customer_repository_1 = require("./finance-customer.repository");
const cuid = require("cuid");
const account_1 = require("../account/account");
const account_repository_1 = require("../account/account.repository");
const typeof_1 = require("../helpers/typeof");
const FinanceDb = require("../database");
class FinanceCustomerBase extends account_system_entity_1.AccountSystemEntity {
    getDetails() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                FullName: this.FullName,
                IDNo: this.IDNo,
                IDType: this.IDType,
                Email: this.Email,
                ContactNo: this.ContactNo,
            };
        });
    }
    constructor(custSystemRefId, custSystemCode, sFinanceCompanyId) {
        super();
        this.CustomerId = 'New';
        this.CustSystemCode = '';
        this.CustSystemRefId = '';
        if (custSystemRefId && custSystemCode && sFinanceCompanyId) {
            const financeCustomerData = FinanceCustomerBase._FinanceCustomerRepository.findOne({
                where: {
                    CompanyId: sFinanceCompanyId,
                    CustSystemCode: custSystemCode,
                    CustSystemRefId: custSystemRefId,
                },
            });
            if (financeCustomerData) {
                this.CompanyId = sFinanceCompanyId;
                this.CustSystemCode = custSystemCode;
                this.CustSystemRefId = custSystemRefId;
            }
            else {
                const notFoundError = new general_1.RecordNotFoundError('No Record Found');
                throw notFoundError;
            }
        }
    }
    static initCustomer(custSystemRefId, custSystemCode, sFinanceCompanyId) {
        return __awaiter(this, void 0, void 0, function* () {
            const financeCustomerData = yield FinanceCustomerBase._FinanceCustomerRepository.findOne({
                where: {
                    CompanyId: sFinanceCompanyId,
                    CustSystemCode: custSystemCode,
                    CustSystemRefId: custSystemRefId,
                },
            });
            if (financeCustomerData) {
                const self = new this();
                self.CustomerId = financeCustomerData.CustomerId;
                self.CompanyId = sFinanceCompanyId;
                self.CustSystemCode = custSystemCode;
                self.CustSystemRefId = custSystemRefId;
                self.AccSystemRefId = financeCustomerData.AccSystemRefId;
                self.PostedToAccSystemYN = financeCustomerData.PostedToAccSystemYN;
                self.PostedById = financeCustomerData.PostedById;
                self.PostedDateTime = financeCustomerData.PostedDateTime;
                return self;
            }
            else {
                const notFoundError = new general_1.RecordNotFoundError('No Record Found');
                throw notFoundError;
            }
        });
    }
    get AccountReceivable() {
        return new Promise((resolve, reject) => {
            let transaction;
            if (this._AccountReceivable) {
                resolve(this._AccountReceivable);
            }
            else {
                FinanceDb.getConnection()
                    .transaction()
                    .then((t) => {
                    transaction = t;
                    return FinanceCustomerBase._AccountRepository.findOne({
                        where: {
                            AccountType: 'Account Receivable',
                            OwnerId: this.CustomerId,
                            OwnerType: (0, typeof_1.default)(this),
                        },
                    });
                })
                    .then((accountData) => {
                    this._AccountReceivable = new account_1.default(transaction);
                    if (accountData) {
                        this._AccountReceivable.AccountNo = accountData.AccountNo;
                        this._AccountReceivable.ParentAccountNo =
                            accountData.ParentAccountNo;
                        this._AccountReceivable.AccountType = accountData.AccountType;
                        this._AccountReceivable.OwnerId = accountData.OwnerId;
                        this._AccountReceivable.OwnerType = accountData.OwnerType;
                    }
                    else {
                        this._AccountReceivable.AccountNo = this.ARAccountNo;
                        this._AccountReceivable.ParentAccountNo = this.ParentARAccountNo;
                        this._AccountReceivable.AccountType = 'Account Receivable';
                        this._AccountReceivable.OwnerId = this.CustomerId;
                        this._AccountReceivable.OwnerType = (0, typeof_1.default)(this);
                    }
                    resolve(this._AccountReceivable);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
        });
    }
    get AccountPayable() {
        return new Promise((resolve, reject) => {
            let transaction;
            if (this._AccountPayable) {
                resolve(this._AccountPayable);
            }
            else {
                FinanceDb.getConnection()
                    .transaction()
                    .then((t) => {
                    transaction = t;
                    return FinanceCustomerBase._AccountRepository.findOne({
                        where: {
                            AccountType: 'Account Payable',
                            OwnerId: this.CustomerId,
                            OwnerType: (0, typeof_1.default)(this),
                        },
                    });
                })
                    .then((accountData) => {
                    this._AccountPayable = new account_1.default(transaction);
                    if (accountData) {
                        this._AccountPayable.AccountNo = accountData.AccountNo;
                        this._AccountPayable.ParentAccountNo =
                            accountData.ParentAccountNo;
                        this._AccountPayable.AccountType = accountData.AccountType;
                        this._AccountPayable.OwnerId = accountData.OwnerId;
                        this._AccountPayable.OwnerType = accountData.OwnerType;
                    }
                    else {
                        this._AccountPayable.AccountNo = this.APAccountNo;
                        this._AccountPayable.ParentAccountNo = this.ParentAPAccountNo;
                        this._AccountPayable.AccountType = 'Account Payable';
                        this._AccountPayable.OwnerId = this.CustomerId;
                        this._AccountPayable.OwnerType = (0, typeof_1.default)(this);
                    }
                    resolve(this._AccountPayable);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
        });
    }
    init(person) {
        this.FullName = person.FullName;
        this.IDNo = person.IDNo;
        this.IDType = person.IDType;
        this.Email = person.Email;
        this.ContactNo = person.ContactNo;
        this.DefaultAddress = person.DefaultAddress;
    }
    save(accSystemRefId, custSystemCode, custSystemRefId, dbTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            this.CustSystemCode = custSystemCode;
            this.CustSystemRefId = custSystemRefId;
            this.AccSystemRefId = accSystemRefId;
            this.PostedToAccSystemYN = 'Y';
            const data = yield FinanceCustomerBase._FinanceCustomerRepository.create({
                CompanyId: this.CompanyId,
                CustomerId: cuid(),
                CustSystemCode: this.CustSystemCode,
                CustSystemRefId: this.CustSystemRefId,
                AccSystemRefId: this.AccSystemRefId,
                PostedToAccSystemYN: this.PostedToAccSystemYN,
            }, { transaction: dbTransaction });
            return data;
        });
    }
    static isFinanceCustomerExist(custSystemRefId, custSystemCode, sFinanceCompanyId) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield FinanceCustomerBase._FinanceCustomerRepository.findAll({
                where: {
                    CustSystemRefId: custSystemRefId,
                    CustSystemCode: custSystemCode,
                    CompanyId: sFinanceCompanyId,
                },
            });
            if (data.length > 0) {
                return true;
            }
            return false;
        });
    }
}
exports.default = FinanceCustomerBase;
FinanceCustomerBase._AccountRepository = new account_repository_1.AccountRepository();
FinanceCustomerBase._FinanceCustomerRepository = new finance_customer_repository_1.FinanceCustomerRepository();
//# sourceMappingURL=customer.js.map