"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const fs = require("fs");
const puppeteer = require("puppeteer");
const general_1 = require("@tomei/general");
const media_1 = require("@tomei/media");
const enum_1 = require("../enum");
const document_item_repository_1 = require("./document-item.repository");
const document_item_1 = require("./document-item");
const account_system_entity_1 = require("../account-system-entity/account-system-entity");
const document_repository_1 = require("./document.repository");
const path = require("path");
class Document extends account_system_entity_1.AccountSystemEntity {
    get IsNewRecord() {
        return this._IsNewRecord;
    }
    set IsNewRecord(record) {
        this._IsNewRecord = record;
    }
    get DocType() {
        return this._DocType;
    }
    set DocType(docType) {
        this._DocType = docType;
    }
    get DocDate() {
        return this._DocDate;
    }
    set DocDate(date) {
        this._DocDate = date;
    }
    get DocHTMLFileMediaId() {
        return this._DocHTMLFileMediaId;
    }
    set DocHTMLFileMediaId(mediaId) {
        this._DocHTMLFileMediaId = mediaId;
    }
    get DocPDFFileMediaId() {
        return this._DocPDFFileMediaId;
    }
    set DocPDFFileMediaId(mediaId) {
        this._DocPDFFileMediaId = mediaId;
    }
    get Amount() {
        return this._Amount;
    }
    set Amount(amount) {
        this._Amount = amount;
    }
    get Status() {
        return this._Status;
    }
    set Status(status) {
        this._Status = status;
    }
    get RepositoryBase() {
        return Document._RepositoryBase;
    }
    get ObjectId() {
        return this.DocNo;
    }
    get ObjectName() {
        return this.DocNo;
    }
    get TableName() {
        return 'finance_Document';
    }
    constructor(dbTransaction, docNo) {
        super();
        this.DocNo = 'New';
        this._DocDate = new Date();
        this.Currency = 'MYR';
        this._Amount = 0;
        this.Description = '';
        this._Status = enum_1.DocumentStatus.UNPAID;
        this.IssuedById = '';
        this.IssuedToId = '';
        this.IssuedToType = '';
        this.RelatedObjectId = '';
        this.RelatedObjectType = '';
        this.CreatedById = '';
        this.UpdatedById = '';
        this.UseAccSystemDocYN = 'N';
        this._DocHTMLFileMediaId = '';
        this._DocPDFFileMediaId = '';
        this._IsNewRecord = true;
        this._DocumentItems = [];
        if (dbTransaction) {
            this._DbTransaction = dbTransaction;
        }
        if (docNo) {
            this.DocNo = docNo;
            this.RepositoryBase.findOne({
                where: {
                    DocNo: this.DocNo,
                },
                transaction: dbTransaction,
            })
                .then((documentData) => {
                if (documentData) {
                    this.IsNewRecord = false;
                    this.DocType = documentData.DocType;
                    this.DocDate = documentData.DocDate;
                    this.CompanyId = documentData.CompanyId;
                    this.Currency = documentData.Currency;
                    this.Amount = documentData.Amount;
                    this.Description = documentData.Description;
                    this.Status = documentData.Status;
                    this.IssuedById = documentData.IssuedById;
                    this.IssuedToId = documentData.IssuedToId;
                    this.IssuedToType = documentData.IssuedToType;
                    this.RelatedObjectId = documentData.RelatedObjectId;
                    this.RelatedObjectType = documentData.RelatedObjectType;
                    this.CreatedById = documentData.CreatedById;
                    this.CreatedAt = documentData.CreatedAt;
                    this.UpdatedById = documentData.UpdatedById;
                    this.UpdatedAt = documentData.UpdatedAt;
                    this.DocPDFFileMediaId = documentData.DocPDFFileMediaId;
                    this.DocHTMLFileMediaId = documentData.DocHTMLFileMediaId;
                    this.AccSystemRefId = documentData.AccSystemRefId;
                    this.PostedToAccSystemYN = documentData.PostedToAccSystemYN;
                    this.PostedById = documentData.PostedById;
                    this.PostedDateTime = documentData.PostedDateTime;
                    this.UseAccSystemDocYN = documentData.UseAccSystemDocYN;
                    this.IsNewRecord = false;
                }
                else {
                    const notFoundError = new general_1.RecordNotFoundError('No Record Found.');
                    throw notFoundError;
                }
            })
                .catch((err) => {
                console.log('Document Class constructor err: ', err);
            });
        }
    }
    static initDocument(dbTransaction, docNo) {
        return __awaiter(this, void 0, void 0, function* () {
            const document = new Document(dbTransaction);
            const documentData = yield this._RepositoryBase.findOne({
                where: {
                    DocNo: docNo,
                },
            });
            if (documentData) {
                document.IsNewRecord = false;
                document.DocType = documentData.DocType;
                document.DocDate = documentData.DocDate;
                document.CompanyId = documentData.CompanyId;
                document.Currency = documentData.Currency;
                document.Amount = documentData.Amount;
                document.Description = documentData.Description;
                document.Status = documentData.Status;
                document.IssuedById = documentData.IssuedById;
                document.IssuedToId = documentData.IssuedToId;
                document.IssuedToType = documentData.IssuedToType;
                document.RelatedObjectId = documentData.RelatedObjectId;
                document.RelatedObjectType = documentData.RelatedObjectType;
                document.CreatedById = documentData.CreatedById;
                document.CreatedAt = documentData.CreatedAt;
                document.UpdatedById = documentData.UpdatedById;
                document.UpdatedAt = documentData.UpdatedAt;
                document.DocPDFFileMediaId = documentData.DocPDFFileMediaId;
                document.DocHTMLFileMediaId = documentData.DocHTMLFileMediaId;
                document.AccSystemRefId = documentData.AccSystemRefId;
                document.PostedToAccSystemYN = documentData.PostedToAccSystemYN;
                document.PostedById = documentData.PostedById;
                document.PostedDateTime = documentData.PostedDateTime;
                document.UseAccSystemDocYN = documentData.UseAccSystemDocYN;
                document.IsNewRecord = false;
            }
            else {
                const notFoundError = new general_1.RecordNotFoundError('No Record Found.');
                throw notFoundError;
            }
            return document;
        });
    }
    reCalculateAmount() {
        return __awaiter(this, void 0, void 0, function* () {
            this.Amount = 0;
            for (const documentItem of this._DocumentItems) {
                this.Amount += documentItem.Amount;
            }
        });
    }
    get DocumentItems() {
        return new Promise((resolve, reject) => {
            if (!this.IsNewRecord) {
                Document._DocumentItemRepository
                    .findAll({
                    where: {
                        DocNo: this.DocNo,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((documentItems) => {
                    const documentItemObjects = documentItems.map((documentItem) => {
                        const di = new document_item_1.default(this._DbTransaction, this);
                        di.init(Object.assign({}, documentItem.get({ plain: true })));
                    });
                    return Promise.all(documentItemObjects);
                })
                    .then((documentItemObjects) => {
                    this._DocumentItems = documentItemObjects;
                    resolve(this._DocumentItems);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            else {
                resolve(this._DocumentItems);
            }
        });
    }
    static DEFAULT_INVOICE_TEMPLATE_HTML() {
        return __awaiter(this, void 0, void 0, function* () {
            const templateLocation = path.resolve('./invoice-template/index.html');
            const readFileContent = util.promisify(fs.readFile);
            const htmlBuffer = yield readFileContent(templateLocation);
            const htmlStream = {
                buffer: htmlBuffer,
                fieldname: 'FileStream',
                originalname: 'index.html',
                mimetype: 'text/html',
                size: htmlBuffer.length,
                encoding: '8bit',
                filename: 'sample-invoice.html',
                destination: '',
                path: '',
                stream: null,
            };
            return htmlStream;
        });
    }
    static DEFAULT_INVOICE_TEMPLATE_PDF() {
        return __awaiter(this, void 0, void 0, function* () {
            const templateLocation = path.resolve('./invoice-template/index.html');
            const readFileContent = util.promisify(fs.readFile);
            const htmlString = yield readFileContent(templateLocation, 'utf8');
            const browser = yield puppeteer.launch();
            const page = yield browser.newPage();
            if (htmlString) {
                yield page.setContent(htmlString);
            }
            else if (templateLocation) {
                yield page.goto(templateLocation, {
                    waitUntil: 'networkidle2',
                });
            }
            else {
                yield browser.close();
            }
            const pdfBuffer = yield page.pdf({
                format: 'a4',
                printBackground: true,
                margin: {
                    left: '0px',
                    top: '0px',
                    right: '0px',
                    bottom: '0px',
                },
            });
            const pdfStream = {
                buffer: pdfBuffer,
                fieldname: 'FileStream',
                originalname: 'sample-invoice.pdf',
                mimetype: 'application/pdf',
                size: pdfBuffer.length,
                encoding: '8bit',
                filename: 'sample-invoice.pdf',
                destination: '',
                path: '',
                stream: null,
            };
            return pdfStream;
        });
    }
    get DocumentFileInHTML() {
        return (() => __awaiter(this, void 0, void 0, function* () {
            const { HTMLMedia } = yield this.generateInvoice();
            return HTMLMedia;
        }))();
    }
    get DocumentFileInPDF() {
        return (() => __awaiter(this, void 0, void 0, function* () {
            const { PDFMedia } = yield this.generateInvoice();
            return PDFMedia;
        }))();
    }
    init(params) {
        this.DocNo = params.DocNo;
        this.DocType = params.DocType;
        this.DocDate = params.DocDate;
        this.CompanyId = params.CompanyId;
        this.Currency = params.Currency;
        this.Amount = params.Amount;
        this.Description = params.Description;
        this.Status = params.Status;
        this.IssuedById = params.IssuedById;
        this.IssuedToId = params.IssuedToId;
        this.IssuedToType = params.IssuedToType;
        this.RelatedObjectId = params.RelatedObjectId;
        this.RelatedObjectType = params.RelatedObjectType;
        this.CreatedById = params.CreatedById;
        this.CreatedAt = params.CreatedAt;
        this.UpdatedById = params.UpdatedById;
        this.UpdatedAt = params.UpdatedAt;
        this.DocPDFFileMediaId = params.DocPDFFileMediaId;
        this.DocHTMLFileMediaId = params.DocHTMLFileMediaId;
        this.AccSystemRefId = params.AccSystemRefId;
        this.PostedToAccSystemYN = params.PostedToAccSystemYN;
        this.PostedById = params.PostedById;
        this.PostedDateTime = params.PostedDateTime;
        this.UseAccSystemDocYN = params.UseAccSystemDocYN;
    }
    generateInvoice(userId, customer) {
        return __awaiter(this, void 0, void 0, function* () {
            const media = new media_1.Medias(Document._MediaRepository, this.mediaCommonService);
            console.log(customer || 'no customer provided', '<< customer');
            const htmlPayload = {
                ObjectId: this.DocNo,
                ObjectType: this.DocType,
                Type: media_1.MediaType.Document,
                FileName: `${this.DocType}-${this.DocNo}`,
                FileStream: yield Document.DEFAULT_INVOICE_TEMPLATE_HTML(),
                FileExtension: 'html',
                Title: `${this.DocType}-${this.DocNo}.html`,
                Description: `HTML ${this.DocType}`,
                IsEncryptedYN: 'N',
            };
            const HTMLInvoiceMedia = yield media.postInternal(yield Document.DEFAULT_INVOICE_TEMPLATE_HTML(), htmlPayload, userId !== null && userId !== void 0 ? userId : 'System');
            const pdfPayload = {
                ObjectId: this.DocNo,
                ObjectType: this.DocType,
                Type: media_1.MediaType.Document,
                FileName: `${this.DocType}-${this.DocNo}`,
                FileStream: yield Document.DEFAULT_INVOICE_TEMPLATE_PDF(),
                FileExtension: 'pdf',
                Title: `${this.DocType}-${this.DocNo}.pdf`,
                Description: `PDF ${this.DocType}`,
                IsEncryptedYN: 'N',
            };
            const PDFInvoiceMedia = yield media.postInternal(yield Document.DEFAULT_INVOICE_TEMPLATE_PDF(), pdfPayload, userId !== null && userId !== void 0 ? userId : 'System');
            try {
                const document = yield this.RepositoryBase.findOne({
                    where: {
                        DocNo: this.DocNo,
                    },
                    transaction: this._DbTransaction,
                });
                document.DocHTMLFileMediaId = HTMLInvoiceMedia.MediaId;
                document.DocPDFFileMediaId = PDFInvoiceMedia.MediaId;
                yield document.save({ transaction: this._DbTransaction });
            }
            catch (err) {
                console.log(err);
            }
            return {
                HTMLMedia: HTMLInvoiceMedia,
                PDFMedia: PDFInvoiceMedia,
            };
        });
    }
    generateCreditNote(userId, customer) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(userId, customer, '<< generateCreditNote');
        });
    }
    newDocumentItem(documentItem) {
        return __awaiter(this, void 0, void 0, function* () {
            let di;
            if (documentItem) {
                di = documentItem;
            }
            else {
                di = new document_item_1.default(this._DbTransaction, this);
            }
            di.DocNo = this.DocNo;
            this._DocumentItems.push(di);
            return documentItem;
        });
    }
    getPreviousDocument(docType) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.RepositoryBase.findOne({
                where: {
                    DocType: docType,
                },
                order: [['CreatedAt', 'DESC']],
            });
        });
    }
}
exports.default = Document;
Document._RepositoryBase = new document_repository_1.DocumentRepository();
Document._DocumentItemRepository = new document_item_repository_1.DocumentItemRepository();
Document._MediaRepository = new media_1.MediasRepository(media_1.MediasModel);
//# sourceMappingURL=document.js.map