"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const cuid = require("cuid");
const general_1 = require("@tomei/general");
const account_1 = require("../account/account");
const journal_entry_1 = require("../journal-entry/journal-entry");
const finance_company_repository_1 = require("./finance-company.repository");
const finance_customer_repository_1 = require("../customer/finance-customer.repository");
const ledger_transaction_repository_1 = require("../ledger-transaction/ledger-transaction.repository");
const enum_1 = require("../enum");
const payment_method_type_1 = require("../payment-method-type/payment-method-type");
const payment_repository_1 = require("../payment/payment.repository");
const payment_item_repository_1 = require("../payment-item/payment-item.repository");
const document_repository_1 = require("../document/document.repository");
const document_item_repository_1 = require("../document/document-item.repository");
const payment_method_repository_1 = require("../payment-method/payment-method.repository");
const payment_method_type_repository_1 = require("../payment-method-type/payment-method-type.repository");
const payment_method_1 = require("../payment-method/payment-method");
const account_repository_1 = require("../account/account.repository");
const payment_paid_with_repository_1 = require("../payment-paid-with/payment-paid-with.repository");
const getConfig = require('../config');
const config = getConfig();
class FinanceCompany extends general_1.ObjectBase {
    get CompSystemCode() {
        return this._CompSystemCode;
    }
    set CompSystemCode(code) {
        this._CompSystemCode = code;
    }
    get CompSystemRefId() {
        return this._CompSystemRefId;
    }
    set CompSystemRefId(id) {
        this._CompSystemRefId = id;
    }
    get AccSystemCode() {
        return this._AccSystemCode;
    }
    set AccSystemCode(code) {
        this._AccSystemCode = code;
    }
    get CompanyId() {
        return this._CompanyId;
    }
    get ObjectId() {
        return this._CompanyId;
    }
    set ObjectId(id) {
        this._CompanyId = id;
    }
    get ObjectName() {
        return `${this.CompSystemCode}-${this.CompSystemRefId}-${this.AccSystemCode}`;
    }
    get TableName() {
        return 'finance_Company';
    }
    get AccountingSystem() {
        return this._AccountingSystem;
    }
    set AccountingSystem(system) {
        this._AccountingSystem = system;
    }
    constructor(compSystemCode, compSystemRefId, accSystemCode) {
        super();
        this._CompanyId = 'New';
        this._CompSystemCode = '';
        this._CompSystemRefId = '';
        this._AccSystemCode = '';
        this._PaymentMethods = [];
        this.CompSystemCode = compSystemCode;
        this.CompSystemRefId = compSystemRefId;
        this.AccSystemCode = accSystemCode;
    }
    static getFinanceCompanyId(compSystemCode, compSystemRefId, accSystemCode) {
        return __awaiter(this, void 0, void 0, function* () {
            let sCompanyId = '';
            const sKey = `${compSystemCode}-${compSystemRefId}-${accSystemCode}`;
            if (!FinanceCompany._htFinanceCompanyIds.get(sKey)) {
                const financeCompany = new FinanceCompany(compSystemCode, compSystemRefId, accSystemCode);
                const company = yield FinanceCompany._financeCompanyRepository.findOne({
                    where: {
                        CompSystemCode: compSystemCode,
                        CompSystemRefId: compSystemRefId,
                        AccSystemCode: accSystemCode,
                    },
                });
                financeCompany.ObjectId = company.CompanyId;
                sCompanyId = financeCompany.ObjectId;
                FinanceCompany._htFinanceCompanyIds.add(sKey, financeCompany.ObjectId);
                FinanceCompany._htFinanceCompanies.add(sCompanyId, financeCompany);
            }
            if (typeof FinanceCompany._htFinanceCompanyIds.get(sKey) === 'string') {
                sCompanyId = FinanceCompany._htFinanceCompanyIds.get(sKey);
            }
            return sCompanyId;
        });
    }
    static getFinanceCompany(companyId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!FinanceCompany._htFinanceCompanies.get(companyId)) {
                const company = yield FinanceCompany._financeCompanyRepository.findOne({
                    where: { CompanyId: companyId },
                });
                if (!company) {
                    throw Error('No finance company found. Please create first.');
                }
                const compSystemCode = company.CompSystemCode;
                const compSystemRefId = company.CompSystemRefId;
                const accSystemCode = company.AccSystemCode;
                const financeCompany = new FinanceCompany(compSystemCode, compSystemRefId, accSystemCode);
                financeCompany.ObjectId = company.CompanyId;
                financeCompany._CompanyId = company.CompanyId;
                const sKey = `${compSystemCode}-${compSystemRefId}-${accSystemCode}`;
                FinanceCompany._htFinanceCompanyIds.add(sKey, financeCompany.ObjectId);
                FinanceCompany._htFinanceCompanies.add(financeCompany.ObjectId, financeCompany);
            }
            console.log('return from hash table');
            return FinanceCompany._htFinanceCompanies.get(companyId);
        });
    }
    static createFinanceCompany(dbTransaction, loginUser, companyId, compSystemCode, compSystemRefId, accSystemCode) {
        return __awaiter(this, void 0, void 0, function* () {
            const financeCompany = new FinanceCompany(compSystemCode, compSystemRefId, accSystemCode);
            const company = yield FinanceCompany._financeCompanyRepository.findOne({
                where: {
                    CompSystemCode: compSystemCode,
                    CompSystemRefId: compSystemRefId,
                    AccSystemCode: accSystemCode,
                },
            });
            if (company) {
                throw Error('There is already another Finance Company with the compSystemCode, CompSystemRefId and accSystemCode specified.');
            }
            financeCompany.ObjectId = companyId;
            yield FinanceCompany._financeCompanyRepository.create({
                CompanyId: financeCompany.CompanyId,
                CompSystemCode: financeCompany.CompSystemCode,
                CompSystemRefId: financeCompany.CompSystemRefId,
                AccSystemCode: financeCompany.AccSystemCode,
            }, {
                transaction: dbTransaction,
            });
            const sKey = `${compSystemCode}-${compSystemRefId}-${accSystemCode}`;
            FinanceCompany._htFinanceCompanyIds.add(sKey, financeCompany.ObjectId);
            FinanceCompany._htFinanceCompanies.add(financeCompany.ObjectId, financeCompany);
            console.log('return from hash table');
            return FinanceCompany._htFinanceCompanies.get(companyId);
        });
    }
    createCustomer(dbTransaction, custSystemCode, custSystemRefId, customer) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!custSystemCode || !custSystemRefId) {
                    throw new Error('CustSystemCode and CustomerRefId are required fields.');
                }
                const financeCustomerData = yield FinanceCompany._FinanceCustomerRepository.findOne({
                    where: {
                        CompanyId: this._CompanyId,
                        CustSystemCode: custSystemCode,
                        CustSystemRefId: custSystemRefId,
                    },
                });
                if (financeCustomerData) {
                    throw new Error('Customer already created previously.');
                }
                const customerId = yield this.AccountingSystem.createCustomer({
                    customer,
                });
                customer.CompanyId = this._CompanyId;
                const newCustomer = yield customer.save(customerId, custSystemCode, custSystemRefId, dbTransaction);
                const payload = {
                    Action: 'Create',
                    Activity: 'Finance Customer Created',
                    Description: `Customer (ID: ${newCustomer.CustomerId}) has been created for ${this.AccSystemCode}`,
                    EntityType: 'finance_Customer',
                    EntityValueBefore: JSON.stringify({}),
                    EntityValueAfter: JSON.stringify(newCustomer),
                    PerformedById: 'test',
                    PerformedAt: new Date(),
                    EntityId: newCustomer.CustomerId,
                };
                yield axios_1.default.post(`${process.env.COMMON_API_URL}/activity-histories`, payload);
                return customer;
            }
            catch (error) {
                throw error;
            }
        });
    }
    postJournal(dbTransaction, journalEntry) {
        return __awaiter(this, void 0, void 0, function* () {
            const debitTransactions = yield journalEntry.DebitTransactions;
            const creditTransactions = yield journalEntry.CreditTransactions;
            try {
                if (creditTransactions.length < 1 || (debitTransactions === null || debitTransactions === void 0 ? void 0 : debitTransactions.length) < 1) {
                    throw new Error('There should be at least 1 debit ledger transaction and 1 credit ledger transaction in the journal entry');
                }
                const totalCreditAmount = creditTransactions.reduce((accumulator, currentValue) => accumulator + currentValue.CreditAmount, 0);
                const totalDebitAmount = debitTransactions.reduce((accumulator, currentValue) => accumulator + currentValue.DebitAmount, 0);
                if (totalCreditAmount !== totalDebitAmount) {
                    throw new Error('Credit ledger transaction and debit ledger transaction should the same amount');
                }
                const newJournalEntry = yield journalEntry.save('test', dbTransaction);
                const allLedgerTransactions = yield FinanceCompany._LedgerTransactionRepository.findAll({});
                for (const ledgerTransaction of allLedgerTransactions) {
                    ledgerTransaction.JournalEntryId = newJournalEntry.JournalEntryId;
                    yield ledgerTransaction.save();
                }
                yield this.AccountingSystem.postJournalEntry(newJournalEntry);
                const payload = {
                    Action: 'Create',
                    Activity: 'Post Journal Entry',
                    Description: `Journal Entry (ID: ${newJournalEntry.JournalEntryId}) has been created`,
                    EntityType: 'JournalEntry',
                    EntityValueBefore: JSON.stringify({}),
                    EntityValueAfter: JSON.stringify(newJournalEntry),
                    PerformedById: 'test',
                    PerformedAt: new Date(),
                    EntityId: newJournalEntry.JournalEntryId,
                };
                yield axios_1.default.post(`${process.env.COMMON_API_URL}/activity-histories`, payload);
            }
            catch (error) {
                throw error;
            }
        });
    }
    createAccount(dbTransaction, account) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                account.validateAccountValue();
                let createAccountPayload = {
                    Name: account.Name,
                    AcctNum: account.AccountNo,
                    AccountType: account.AccountType,
                    AccountSubType: account.AccountSubtype,
                };
                if (account.isParentAccountExists()) {
                    createAccountPayload = Object.assign(Object.assign({}, createAccountPayload), { CurrencyRef: 'MYR', ParentRef: account.ParentAccountNo, SubAccount: true });
                }
                const accSystemAccountId = yield this.AccountingSystem.createAccount(createAccountPayload);
                const newAccount = yield account.save(accSystemAccountId, 'test', dbTransaction);
                const payload = {
                    Action: 'Create',
                    Activity: 'Account Created',
                    Description: `Account (ID: ${newAccount.AccountNo}) has been created`,
                    EntityType: 'Account',
                    EntityValueBefore: JSON.stringify({}),
                    EntityValueAfter: JSON.stringify(newAccount),
                    PerformedById: 'test',
                    PerformedAt: new Date(),
                    EntityId: newAccount.AccountNo,
                };
                yield axios_1.default.post(`${process.env.COMMON_API_URL}/activity-histories`, payload);
                return account;
            }
            catch (error) {
                throw error;
            }
        });
    }
    issueInvoice(dbTransaction, invoice, loginUser, customer, dtAccountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const duplicateInvoice = yield FinanceCompany._DocumentRepository.findOne({
                    where: {
                        DocNo: invoice.DocNo,
                    },
                    transaction: dbTransaction,
                });
                if (duplicateInvoice) {
                    throw new Error('Invoice number already exists');
                }
                const documentItems = yield invoice.DocumentItems;
                if (!documentItems.length) {
                    throw new Error('Document must have at least 1 document item');
                }
                for (const invoiceItem of documentItems) {
                    if (!invoiceItem.CtAccountNo) {
                        throw new Error('Each document item should have CtAccountNo provided');
                    }
                }
                invoice.DocType = enum_1.DocType.INVOICE;
                yield FinanceCompany._DocumentRepository.create({
                    DocNo: invoice.DocNo,
                    DocType: invoice.DocType,
                    DocDate: invoice.DocDate,
                    CompanyId: invoice.CompanyId,
                    Currency: invoice.Currency,
                    Amount: invoice.Amount,
                    Description: invoice.Description,
                    Status: invoice.Status,
                    IssuedById: invoice.IssuedById,
                    IssuedToId: invoice.IssuedToId,
                    IssuedToType: invoice.IssuedToType,
                    RelatedObjectId: invoice.RelatedObjectId,
                    RelatedObjectType: invoice.RelatedObjectType,
                    CreatedById: invoice.CreatedById,
                    CreatedAt: new Date(),
                    UpdatedById: invoice.UpdatedById,
                    UpdatedAt: new Date(),
                    DocPDFFileMediaId: invoice.DocPDFFileMediaId,
                    DocHTMLFileMediaId: invoice.DocHTMLFileMediaId,
                    AccSystemRefId: invoice.AccSystemRefId,
                    PostedToAccSystemYN: invoice.PostedToAccSystemYN,
                    PostedById: invoice.PostedById,
                    PostedDateTime: new Date(),
                    UseAccSystemDocYN: invoice.UseAccSystemDocYN,
                }, {
                    transaction: dbTransaction,
                });
                for (const documentItem of documentItems) {
                    yield FinanceCompany._DocumentItemRepository.create({
                        DocumentItemId: cuid(),
                        DocNo: invoice.DocNo,
                        Name: documentItem.Name,
                        NameBM: documentItem.NameBM,
                        Description: documentItem.Description,
                        ItemId: documentItem.ItemId,
                        ItemType: documentItem.ItemType,
                        ItemSKU: documentItem.ItemSKU,
                        ItemSerialNo: documentItem.ItemSerialNo,
                        Currency: documentItem.Currency,
                        UnitPrice: documentItem.UnitPrice,
                        Quantity: documentItem.Quantity,
                        QuantityUOM: documentItem.QuantityUOM,
                        Amount: documentItem.Amount,
                        TaxCode: documentItem.TaxCode,
                        TaxAmount: documentItem.TaxAmount,
                        TaxRate: documentItem.TaxRate,
                        TaxInclusiveYN: documentItem.TaxInclusiveYN,
                        DtAccountNo: documentItem.DtAccountNo
                            ? documentItem.DtAccountNo
                            : null,
                        CtAccountNo: documentItem.CtAccountNo
                            ? documentItem.CtAccountNo
                            : null,
                    }, {
                        transaction: dbTransaction,
                    });
                }
                if (invoice.UseAccSystemDocYN === 'Y') {
                    yield this.AccountingSystem.createInvoice(invoice);
                }
                else {
                    invoice.generateInvoice(invoice.IssuedById, customer);
                }
                const transactionDate = new Date();
                const htCreditAccountAmount = new general_1.HashTable();
                const htCreditAccountCurrency = new general_1.HashTable();
                documentItems.forEach((invoiceItem) => {
                    if (!htCreditAccountAmount.exists(invoiceItem.CtAccountNo)) {
                        htCreditAccountAmount.add(invoiceItem.CtAccountNo, invoiceItem.Amount);
                        htCreditAccountCurrency.add(invoiceItem.CtAccountNo, invoiceItem.Currency);
                    }
                    else {
                        const d = htCreditAccountAmount.get(invoiceItem.CtAccountNo);
                        htCreditAccountAmount.add(invoiceItem.CtAccountNo, d + invoiceItem.Amount);
                    }
                });
                const savedItems = htCreditAccountAmount.list();
                for (let i = 0; i < savedItems.length; i++) {
                    const journalEntry = new journal_entry_1.default(dbTransaction);
                    journalEntry.init({
                        CompanyId: this.CompanyId,
                        Name: 'Issue Invoice ' + invoice.DocNo,
                    });
                    const account = yield account_1.default.initAccount(dbTransaction, savedItems[i].value[0]);
                    const creditAmount = savedItems[i].value[1];
                    const currency = htCreditAccountCurrency.get(savedItems[i].value[0]);
                    const dt = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.DEBIT);
                    if (dtAccountNo) {
                        dt.AccountNo = dtAccountNo;
                    }
                    else {
                        dt.AccountNo = (yield customer.AccountReceivable).AccountNo;
                    }
                    dt.Currency = currency ? currency : 'MYR';
                    dt.DebitAmount = creditAmount ? creditAmount : 0.0;
                    dt.Date = transactionDate;
                    dt.Description = account.Name;
                    dt.Name = account.Name;
                    dt.RelatedDocNo = invoice.DocNo;
                    const ct = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.CREDIT);
                    ct.AccountNo = savedItems[i].key;
                    ct.Currency = currency ? currency : 'MYR';
                    ct.CreditAmount = creditAmount ? creditAmount : 0.0;
                    ct.Date = transactionDate;
                    ct.Description = account.Name;
                    ct.Name = account.Name;
                    ct.RelatedDocNo = invoice.DocNo;
                    yield this.postJournal(dbTransaction, journalEntry);
                }
                return invoice;
            }
            catch (err) {
                console.log('Issue invoice err: ', err);
                throw err;
            }
        });
    }
    issueDebitNote(dbTransaction, loginUser, invoice, customer, dtAccountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const duplicateInvoice = yield FinanceCompany._DocumentRepository.findOne({
                    where: {
                        DocNo: invoice.DocNo,
                    },
                    transaction: dbTransaction,
                });
                if (duplicateInvoice) {
                    throw new Error('Invoice number already exists');
                }
                const documentItems = yield invoice.DocumentItems;
                if (!documentItems.length) {
                    throw new Error('Document must have at least 1 document item');
                }
                for (const invoiceItem of documentItems) {
                    if (!invoiceItem.CtAccountNo) {
                        throw new Error('Each document item should have CtAccountNo provided');
                    }
                }
                invoice.DocType = enum_1.DocType.DEBIT_NOTE;
                yield FinanceCompany._DocumentRepository.create({
                    DocNo: invoice.DocNo,
                    DocType: invoice.DocType,
                    DocDate: invoice.DocDate,
                    CompanyId: invoice.CompanyId,
                    Currency: invoice.Currency,
                    Amount: invoice.Amount,
                    Description: invoice.Description,
                    Status: invoice.Status,
                    IssuedById: loginUser.ObjectId,
                    IssuedToId: customer.ObjectId,
                    IssuedToType: invoice.IssuedToType,
                    RelatedObjectId: invoice.RelatedObjectId,
                    RelatedObjectType: invoice.RelatedObjectType,
                    CreatedById: loginUser.ObjectId,
                    CreatedAt: new Date(),
                    UpdatedById: loginUser.ObjectId,
                    UpdatedAt: new Date(),
                    DocPDFFileMediaId: invoice.DocPDFFileMediaId,
                    DocHTMLFileMediaId: invoice.DocHTMLFileMediaId,
                    AccSystemRefId: invoice.AccSystemRefId,
                    PostedToAccSystemYN: invoice.PostedToAccSystemYN,
                    PostedById: invoice.PostedById,
                    PostedDateTime: new Date(),
                    UseAccSystemDocYN: invoice.UseAccSystemDocYN,
                }, {
                    transaction: dbTransaction,
                });
                documentItems.forEach((documentItem) => __awaiter(this, void 0, void 0, function* () {
                    yield FinanceCompany._DocumentItemRepository.create({
                        DocumentItemId: cuid(),
                        DocNo: documentItem.DocNo,
                        Name: documentItem.Name,
                        NameBM: documentItem.NameBM,
                        Description: documentItem.Description,
                        ItemId: documentItem.ItemId,
                        ItemType: documentItem.ItemType,
                        ItemSKU: documentItem.ItemSKU,
                        ItemSerialNo: documentItem.ItemSerialNo,
                        Currency: documentItem.Currency,
                        UnitPrice: documentItem.UnitPrice,
                        Quantity: documentItem.Quantity,
                        QuantityUOM: documentItem.QuantityUOM,
                        Amount: documentItem.Amount,
                        TaxCode: documentItem.TaxCode,
                        TaxAmount: documentItem.TaxAmount,
                        TaxRate: documentItem.TaxRate,
                        TaxInclusiveYN: documentItem.TaxInclusiveYN,
                        DtAccountNo: documentItem.DtAccountNo,
                        CtAccountNo: documentItem.CtAccountNo,
                    }, {
                        transaction: dbTransaction,
                    });
                }));
                if (invoice.UseAccSystemDocYN === 'Y') {
                    yield this.AccountingSystem.createInvoice(invoice);
                }
                else {
                    invoice.generateInvoice(loginUser.IDNo, customer);
                }
                const transactionDate = new Date();
                const htCreditAccountAmount = new general_1.HashTable();
                const htCreditAccountCurrency = new general_1.HashTable();
                documentItems.forEach((invoiceItem) => {
                    if (!htCreditAccountAmount.exists(invoiceItem.CtAccountNo)) {
                        htCreditAccountAmount.add(invoiceItem.CtAccountNo, invoiceItem.Amount);
                        htCreditAccountCurrency.add(invoiceItem.CtAccountNo, invoiceItem.Currency);
                    }
                    else {
                        const d = htCreditAccountAmount.get(invoiceItem.CtAccountNo);
                        htCreditAccountAmount.add(invoiceItem.CtAccountNo, d + invoiceItem.Amount);
                    }
                });
                const savedItems = htCreditAccountAmount.list();
                for (let i = 0; i < savedItems.length; i++) {
                    const journalEntry = new journal_entry_1.default(dbTransaction);
                    journalEntry.init({
                        CompanyId: this.CompanyId,
                        Name: 'issue Invoice ' + invoice.DocNo,
                    });
                    const account = yield account_1.default.initAccount(dbTransaction, savedItems[i].value[0]);
                    const creditAmount = savedItems[i].value[1];
                    const currency = htCreditAccountCurrency.get(savedItems[i].value[0]);
                    const dt = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.DEBIT);
                    if (dtAccountNo) {
                        dt.AccountNo = dtAccountNo;
                    }
                    else {
                        dt.AccountNo = (yield customer.AccountReceivable).AccountNo;
                    }
                    dt.Currency = currency ? currency : 'MYR';
                    dt.DebitAmount = creditAmount ? creditAmount : 0.0;
                    dt.Date = transactionDate;
                    dt.Description = account.Name;
                    dt.Name = account.Name;
                    dt.RelatedDocNo = invoice.DocNo;
                    const ct = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.CREDIT);
                    ct.AccountNo = savedItems[i].key;
                    ct.Currency = currency ? currency : 'MYR';
                    ct.CreditAmount = creditAmount ? creditAmount : 0.0;
                    ct.Date = transactionDate;
                    ct.Description = account.Name;
                    ct.Name = account.Name;
                    ct.RelatedDocNo = invoice.DocNo;
                    yield this.postJournal(dbTransaction, journalEntry);
                }
                return invoice;
            }
            catch (err) {
                console.log('Issue debit note err: ', err);
                throw err;
            }
        });
    }
    issueCreditNote(dbTransaction, loginUser, creditNote, customer, ctAccountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const duplicateCreditNote = yield FinanceCompany._DocumentRepository.findOne({
                    where: {
                        DocNo: creditNote.DocNo,
                    },
                    transaction: dbTransaction,
                });
                if (duplicateCreditNote) {
                    throw new Error('Invoice number already exists');
                }
                const documentItems = yield creditNote.DocumentItems;
                if (!documentItems.length) {
                    throw new Error('Document must have at least 1 document item');
                }
                for (const invoiceItem of documentItems) {
                    if (!invoiceItem.CtAccountNo) {
                        throw new Error('Each document item should have CtAccountNo provided');
                    }
                }
                creditNote.DocType = enum_1.DocType.DEBIT_NOTE;
                yield FinanceCompany._DocumentRepository.create({
                    DocNo: creditNote.DocNo,
                    DocType: creditNote.DocType,
                    DocDate: creditNote.DocDate,
                    CompanyId: creditNote.CompanyId,
                    Currency: creditNote.Currency,
                    Amount: creditNote.Amount,
                    Description: creditNote.Description,
                    Status: creditNote.Status,
                    IssuedById: loginUser.ObjectId,
                    IssuedToId: customer.ObjectId,
                    IssuedToType: creditNote.IssuedToType,
                    RelatedObjectId: creditNote.RelatedObjectId,
                    RelatedObjectType: creditNote.RelatedObjectType,
                    CreatedById: loginUser.ObjectId,
                    CreatedAt: new Date(),
                    UpdatedById: loginUser.ObjectId,
                    UpdatedAt: new Date(),
                    DocPDFFileMediaId: creditNote.DocPDFFileMediaId,
                    DocHTMLFileMediaId: creditNote.DocHTMLFileMediaId,
                    AccSystemRefId: creditNote.AccSystemRefId,
                    PostedToAccSystemYN: creditNote.PostedToAccSystemYN,
                    PostedById: creditNote.PostedById,
                    PostedDateTime: new Date(),
                    UseAccSystemDocYN: creditNote.UseAccSystemDocYN,
                }, {
                    transaction: dbTransaction,
                });
                documentItems.forEach((documentItem) => __awaiter(this, void 0, void 0, function* () {
                    yield FinanceCompany._DocumentItemRepository.create({
                        DocumentItemId: documentItem.DocumentItemId,
                        DocNo: documentItem.DocNo,
                        Name: documentItem.Name,
                        NameBM: documentItem.NameBM,
                        Description: documentItem.Description,
                        ItemId: documentItem.ItemId,
                        ItemType: documentItem.ItemType,
                        ItemSKU: documentItem.ItemSKU,
                        ItemSerialNo: documentItem.ItemSerialNo,
                        Currency: documentItem.Currency,
                        UnitPrice: documentItem.UnitPrice,
                        Quantity: documentItem.Quantity,
                        QuantityUOM: documentItem.QuantityUOM,
                        Amount: documentItem.Amount,
                        TaxCode: documentItem.TaxCode,
                        TaxAmount: documentItem.TaxAmount,
                        TaxRate: documentItem.TaxRate,
                        TaxInclusiveYN: documentItem.TaxInclusiveYN,
                        DtAccountNo: documentItem.DtAccountNo,
                        CtAccountNo: documentItem.CtAccountNo,
                    }, {
                        transaction: dbTransaction,
                    });
                }));
                if (creditNote.UseAccSystemDocYN === 'Y') {
                    yield this.AccountingSystem.createCreditNote(creditNote);
                }
                else {
                    creditNote.generateCreditNote(loginUser.IDNo, customer);
                }
                const journalEntry = new journal_entry_1.default(dbTransaction);
                journalEntry.init({
                    CompanyId: this.CompanyId,
                    Name: 'Issue Debit Note ' + creditNote.DocNo,
                });
                const transactionDate = new Date();
                const creditTransaction = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.CREDIT);
                if (ctAccountNo) {
                    creditTransaction.AccountNo = ctAccountNo;
                }
                else {
                    creditTransaction.AccountNo = customer.CustSystemCode + '-AP';
                }
                creditTransaction.Currency = creditNote.Currency;
                creditTransaction.CreditAmount = creditNote.Amount;
                creditTransaction.Date = transactionDate;
                creditTransaction.Description = creditNote.DocNo;
                creditTransaction.Name = creditNote.DocNo;
                for (const invoiceItem of documentItems) {
                    const itemLedger = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.DEBIT);
                    itemLedger.AccountNo = invoiceItem.CtAccountNo;
                    itemLedger.Currency = invoiceItem.Currency;
                    itemLedger.DebitAmount = invoiceItem.Amount;
                    itemLedger.Date = transactionDate;
                    itemLedger.Description = invoiceItem.Description;
                    itemLedger.Name = invoiceItem.Name;
                }
                this.postJournal(dbTransaction, journalEntry);
                const payload = {
                    Action: 'Create',
                    Activity: 'Issuing a Credit Note Transaction',
                    Description: `Credit Transaction (ID: ${creditTransaction.TransactionId}) has been created`,
                    EntityType: 'CreditTransaction',
                    EntityValueBefore: JSON.stringify({}),
                    EntityValueAfter: JSON.stringify(creditTransaction),
                    PerformedById: 'test',
                    PerformedAt: transactionDate,
                    EntityId: creditTransaction.TransactionId,
                };
                yield axios_1.default.post(`${process.env.COMMON_API_URL}/activity-histories`, payload);
                return creditNote;
            }
            catch (err) {
                console.log('Issue credit note err: ', err);
                throw err;
            }
        });
    }
    collectPayment(dbTransaction, loginUser, payment, customer, ctAccountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const paymentItems = yield payment.PaymentItems;
                if (paymentItems.length < 1) {
                    throw new Error('Atleast one payment item is required to identify what payment is being paid for.');
                }
                const paymentPaidWithItems = yield payment.PaymentPaidWith;
                if (paymentPaidWithItems.length < 1) {
                    throw new Error('Atleast one payment paid with item is required to identify how the payment was made.');
                }
                payment.PaymentId = cuid();
                payment.PaymentType = enum_1.PaymentType.PAYMENT_RECEIVED;
                payment.ReceivedBy = loginUser.ObjectId;
                yield FinanceCompany._PaymentRepository.create({
                    PaymentId: payment.PaymentId,
                    PaymentType: payment.PaymentType,
                    PaymentDate: payment.PaymentDate,
                    Description: payment.Description,
                    Currency: payment.Currency,
                    Amount: payment.Amount,
                    Status: enum_1.PaymentStatus.SUCCESSFUL,
                    PostedToAccSystemYN: 'N',
                    ReceivedBy: payment.ReceivedBy,
                    UpdatedAt: new Date(),
                    UpdatedBy: loginUser.ObjectId,
                    CreatedAt: new Date(),
                    CreatedBy: loginUser.ObjectId,
                }, { transaction: dbTransaction });
                for (const paymentItem of paymentItems) {
                    yield FinanceCompany._PaymentItemRepository.create({
                        PaymentId: payment.PaymentId,
                        PayForObjectId: paymentItem.PayForObjectId,
                        PayForObjectType: paymentItem.PayForObjectType,
                        Currency: paymentItem.Currency,
                        Amount: paymentItem.Amount,
                        Name: paymentItem.Name,
                        Description: paymentItem.Description,
                    }, { transaction: dbTransaction });
                }
                for (const paymentPaidWithItem of paymentPaidWithItems) {
                    yield FinanceCompany._PaymentPaidWithRepository.create({
                        PaymentId: payment.PaymentId,
                        MethodTypeId: paymentPaidWithItem.MethodTypeId,
                        Currency: paymentPaidWithItem.Currency,
                        Amount: paymentPaidWithItem.Amount,
                        Status: paymentPaidWithItem.Status,
                        TransactionId: paymentPaidWithItem.TransactionId,
                        RefBank: paymentPaidWithItem.RefBank,
                        RefName: paymentPaidWithItem.RefName,
                        RefNo: paymentPaidWithItem.RefNo,
                        RefOther1: paymentPaidWithItem.RefOther1,
                        RefOther2: paymentPaidWithItem.RefOther2,
                        RefOther3: paymentPaidWithItem.RefOther3,
                        RefOther4: paymentPaidWithItem.RefOther4,
                        RefOther5: paymentPaidWithItem.RefOther5,
                        Remarks: paymentPaidWithItem.Remarks,
                        PaymentMediaId: paymentPaidWithItem.PaymentMediaId,
                    }, { transaction: dbTransaction });
                }
                const transactionDate = new Date();
                for (const paymentPaidWith of paymentPaidWithItems) {
                    let paymentMethodType;
                    try {
                        paymentMethodType = new payment_method_type_1.default(dbTransaction, paymentPaidWith.MethodTypeId);
                    }
                    catch (error) {
                        if (error instanceof general_1.RecordNotFoundError) {
                            throw new Error('Invalid Payment Method Type Id');
                        }
                        else {
                            throw error;
                        }
                    }
                    const journalEntry = new journal_entry_1.default(dbTransaction);
                    journalEntry.init({
                        CompanyId: this.CompanyId,
                        Name: 'Collect-payments for ' + payment.PaymentId,
                    });
                    const debitLT = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.DEBIT);
                    debitLT.AccountNo = paymentMethodType.AccountNo;
                    debitLT.Currency = payment.Currency;
                    debitLT.DebitAmount = payment.Amount;
                    debitLT.Date = transactionDate;
                    debitLT.Description = customer.FullName;
                    debitLT.Name = customer.FullName;
                    const creditLT = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.CREDIT);
                    if (ctAccountNo) {
                        creditLT.AccountNo = ctAccountNo;
                    }
                    else {
                        creditLT.AccountNo = (yield customer.AccountReceivable).AccountNo;
                    }
                    creditLT.Currency = payment.Currency;
                    creditLT.CreditAmount = payment.Amount;
                    creditLT.Date = transactionDate;
                    creditLT.Description = paymentMethodType.Name;
                    creditLT.Name = paymentMethodType.Name;
                    yield this.postJournal(dbTransaction, journalEntry);
                }
                return payment;
            }
            catch (error) {
                throw error;
            }
        });
    }
    makePayment(dbTransaction, loginUser, payment, customer, dtAccountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            let paymentMethodType;
            try {
                paymentMethodType = new payment_method_type_1.default(dbTransaction);
                const paymentItems = yield payment.PaymentItems;
                if (paymentItems.length < 1) {
                    throw new Error('Atleast one payment item is required to identify what payment is being paid for');
                }
                payment.PaymentId = cuid();
                payment.PaymentType = enum_1.PaymentType.PAYOUT;
                payment.ReceivedBy = loginUser.ObjectId;
                yield FinanceCompany._PaymentRepository.create({
                    PaymentId: payment.PaymentId,
                    PaymentType: payment.PaymentType,
                    PaymentDate: payment.PaymentDate,
                    Description: payment.Description,
                    Currency: payment.Currency,
                    ReceivedBy: payment.ReceivedBy,
                    Amount: payment.Amount,
                    Status: enum_1.PaymentStatus.SUCCESSFUL,
                    PostedToAccSystemYN: 'N',
                    UpdatedAt: new Date(),
                    UpdatedBy: loginUser.ObjectId,
                    CreatedAt: new Date(),
                    CreatedBy: loginUser.ObjectId,
                }, { transaction: dbTransaction });
                paymentItems.forEach((paymentItem) => __awaiter(this, void 0, void 0, function* () {
                    yield FinanceCompany._PaymentItemRepository.create({
                        PaymentId: payment.PaymentId,
                        PayForObjectId: paymentItem.PayForObjectId,
                        PayForObjectType: paymentItem.PayForObjectType,
                        Currency: paymentItem.Currency,
                        Amount: paymentItem.Amount,
                        Name: paymentItem.Name,
                        Description: paymentItem.Description,
                    }, { transaction: dbTransaction });
                }));
                const journalEntry = new journal_entry_1.default(dbTransaction);
                journalEntry.init({
                    CompanyId: this.CompanyId,
                    Name: 'Make Payment for ' + payment.PaymentId,
                });
                const transactionDate = new Date();
                const creditLT = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.CREDIT);
                creditLT.AccountNo = paymentMethodType.AccountNo;
                creditLT.Currency = payment.Currency;
                creditLT.CreditAmount = payment.Amount;
                creditLT.Date = transactionDate;
                creditLT.Description = customer.FullName;
                creditLT.Name = customer.FullName;
                const debitLT = yield journalEntry.newLedgerTransaction(enum_1.TransactionTypeOptions.DEBIT);
                if (dtAccountNo) {
                    debitLT.AccountNo = dtAccountNo;
                }
                else {
                    debitLT.AccountNo = (yield customer.AccountPayable).AccountNo;
                }
                debitLT.Currency = payment.Currency;
                debitLT.DebitAmount = payment.Amount;
                debitLT.Date = transactionDate;
                debitLT.Description = paymentMethodType.Name;
                debitLT.Name = paymentMethodType.Name;
                yield this.postJournal(dbTransaction, journalEntry);
                return payment;
            }
            catch (error) {
                if (error instanceof general_1.RecordNotFoundError) {
                    throw new Error('Invalid PaymentMethodType id');
                }
                else {
                    throw error;
                }
            }
        });
    }
    get PaymentMethods() {
        return new Promise((resolve, reject) => {
            if (this.CompanyId !== 'New') {
                FinanceCompany._PaymentMethodRepository
                    .findAll({
                    where: {
                        CompanyId: this.CompanyId,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((paymentMethod) => {
                    const paymentMethodObjects = paymentMethod.map((paymentMethodData) => {
                        return new Promise((resolve, reject) => {
                            FinanceCompany._PaymentMethodTypeRepository
                                .findAll({
                                where: {
                                    MethodId: paymentMethodData.MethodId,
                                },
                                transaction: this._DbTransaction,
                                raw: true,
                            })
                                .then((paymentMethodTypes) => {
                                const paymentMethodObjects = Object.assign(Object.assign({}, paymentMethodData.get({ plain: true })), { Types: paymentMethodTypes });
                                resolve(paymentMethodObjects);
                            })
                                .catch((err) => {
                                reject(err);
                            });
                        }).then((paymentMethods) => paymentMethods);
                    });
                    return Promise.all(paymentMethodObjects);
                })
                    .then((paymentMethodObjects) => {
                    this._PaymentMethods = paymentMethodObjects;
                    resolve(this._PaymentMethods);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            else {
                resolve(this._PaymentMethods);
            }
        });
    }
    LoadPaymentMethods() {
        return __awaiter(this, void 0, void 0, function* () {
            const { companyId, paymentMethods } = config.financeCompanies['TXG-FS'];
            const paymentMethod = new payment_method_1.default();
            for (const method in paymentMethods) {
                const paymentMethodData = yield FinanceCompany._PaymentMethodRepository.findOne({
                    where: {
                        MethodId: paymentMethods[method].id,
                        Name: paymentMethods[method].name,
                    },
                });
                if (!paymentMethodData) {
                    const newPaymentMethod = yield FinanceCompany._PaymentMethodRepository.create({
                        MethodId: paymentMethods[method].id,
                        Name: paymentMethods[method].name,
                        CompanyId: companyId,
                    });
                    this._PaymentMethods.push(newPaymentMethod);
                }
                this._PaymentMethods.push(paymentMethodData);
            }
            this._PaymentMethods.forEach((item) => __awaiter(this, void 0, void 0, function* () {
                var _a, _b;
                const p = item === null || item === void 0 ? void 0 : item.get({ plain: true });
                for (const method in paymentMethods) {
                    if (!p) {
                        continue;
                    }
                    if (p.MethodId === ((_a = paymentMethods[method]) === null || _a === void 0 ? void 0 : _a.id)) {
                        const paymentMethodTypeData = yield FinanceCompany._PaymentMethodTypeRepository.findOne({
                            where: {
                                MethodId: p.MethodId,
                            },
                        });
                        if (!paymentMethodTypeData) {
                            const configPaymentMethodTypes = (_b = paymentMethods[method]) === null || _b === void 0 ? void 0 : _b.types;
                            for (const methodType in configPaymentMethodTypes) {
                                const accountData = yield FinanceCompany._AccountRepository.findOne({
                                    where: {
                                        AccountNo: configPaymentMethodTypes[methodType].accountNo,
                                    },
                                });
                                if (!accountData) {
                                    const accountPayload = {
                                        CompanyId: companyId,
                                        Name: configPaymentMethodTypes[methodType].name,
                                        AccountType: 'PaymentMethod',
                                        CreatedAt: new Date(),
                                        CreatedById: 'System',
                                        AccSystemRefId: 'REF',
                                        PostedToAccSystemYN: 'N',
                                    };
                                    try {
                                        yield FinanceCompany._AccountRepository.create(Object.assign({ AccountNo: configPaymentMethodTypes[methodType].accountNo }, accountPayload));
                                        yield FinanceCompany._AccountRepository.create(Object.assign({ AccountNo: configPaymentMethodTypes[methodType]
                                                .processingFeeAccountNo }, accountPayload));
                                    }
                                    catch (err) { }
                                }
                                const paymentMethodTypePayload = {
                                    MethodId: p.MethodId,
                                    MethodTypeId: configPaymentMethodTypes[methodType].id,
                                    Name: configPaymentMethodTypes[methodType].name,
                                    AccountNo: configPaymentMethodTypes[methodType].accountNo,
                                    ProcessingFeeRate: configPaymentMethodTypes[methodType].processingFeeRate,
                                    ProcessingFeeAccountNo: configPaymentMethodTypes[methodType].processingFeeAccountNo,
                                };
                                try {
                                    yield paymentMethod.newPaymentMethodType(paymentMethodTypePayload);
                                }
                                catch (err) { }
                            }
                        }
                    }
                }
            }));
        });
    }
}
exports.default = FinanceCompany;
FinanceCompany._htFinanceCompanyIds = new general_1.HashTable();
FinanceCompany._htFinanceCompanies = new general_1.HashTable();
FinanceCompany._financeCompanyRepository = new finance_company_repository_1.FinanceCompanyRepository();
FinanceCompany._PaymentRepository = new payment_repository_1.PaymentRepository();
FinanceCompany._PaymentItemRepository = new payment_item_repository_1.PaymentItemRepository();
FinanceCompany._PaymentPaidWithRepository = new payment_paid_with_repository_1.PaymentPaidWithRepository();
FinanceCompany._PaymentMethodRepository = new payment_method_repository_1.PaymentMethodRepository();
FinanceCompany._PaymentMethodTypeRepository = new payment_method_type_repository_1.PaymentMethodTypeRepository();
FinanceCompany._DocumentRepository = new document_repository_1.DocumentRepository();
FinanceCompany._DocumentItemRepository = new document_item_repository_1.DocumentItemRepository();
FinanceCompany._FinanceCustomerRepository = new finance_customer_repository_1.FinanceCustomerRepository();
FinanceCompany._LedgerTransactionRepository = new ledger_transaction_repository_1.LedgerTransactionRepository();
FinanceCompany._AccountRepository = new account_repository_1.AccountRepository();
//# sourceMappingURL=finance-company.js.map