"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cuid = require("cuid");
const enum_1 = require("../enum");
const account_system_entity_1 = require("../account-system-entity/account-system-entity");
const ledger_transaction_repository_1 = require("../ledger-transaction/ledger-transaction.repository");
const ledger_transaction_1 = require("../ledger-transaction/ledger-transaction");
const journal_entry_repository_1 = require("./journal-entry.repository");
const general_1 = require("@tomei/general");
class JournalEntry extends account_system_entity_1.AccountSystemEntity {
    get JournalEntryId() {
        return this._JournalEntryId;
    }
    set JournalEntryId(id) {
        this._JournalEntryId = id;
    }
    get RepositoryBase() {
        return JournalEntry._RepositoryBase;
    }
    get TableName() {
        return 'finance_JournalEntry';
    }
    get ObjectName() {
        return this.Name;
    }
    get ObjectId() {
        return this._JournalEntryId;
    }
    constructor(dbTransaction, journalEntryId) {
        super();
        this._JournalEntryId = 'New';
        this.Date = new Date();
        this.Name = '';
        this.Description = '';
        this._DebitTransactions = [];
        this._CreditTransactions = [];
        if (dbTransaction) {
            this._DbTransaction = dbTransaction;
        }
        if (journalEntryId) {
            this.RepositoryBase.findOne({
                where: {
                    JournalEntryId: journalEntryId,
                },
                transaction: dbTransaction,
            })
                .then((journalEntryData) => {
                if (journalEntryData) {
                    this.CompanyId = journalEntryData.CompanyId;
                    this.Date = journalEntryData.Date;
                    this.Name = journalEntryData.Name;
                    this.Description = journalEntryData.Description;
                    this.AccSystemRefId = journalEntryData.AccSystemRefId;
                    this.PostedToAccSystemYN = journalEntryData.PostedToAccSystemYN;
                    this.PostedById = journalEntryData.PostedById;
                    this.PostedDateTime = journalEntryData.PostedDateTime;
                }
                else {
                    const notFoundError = new general_1.RecordNotFoundError('No Record Found.');
                    throw notFoundError;
                }
            })
                .catch((err) => {
                console.log('Journal entry constructor err: ', err);
            });
        }
        else {
            this.init({
                JournalEntryId: 'New',
                CompanyId: '',
                Date: new Date(),
                Name: '',
                Description: '',
                AccSystemRefId: '',
                PostedToAccSystemYN: 'N',
                PostedById: '',
                PostedDateTime: null,
            });
        }
    }
    get DebitTransactions() {
        return new Promise((resolve, reject) => {
            if (this.JournalEntryId !== 'New') {
                JournalEntry._LedgerTransactionRepository
                    .findAll({
                    where: {
                        TransactionType: enum_1.TransactionTypeOptions.DEBIT,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((debitTransactions) => {
                    const debitTransactionObjects = debitTransactions.map((debitTransactionData) => new ledger_transaction_1.default(this._DbTransaction, debitTransactionData.TransactionId));
                    return Promise.all(debitTransactionObjects);
                })
                    .then((debitTransactionObjects) => {
                    this._DebitTransactions = debitTransactionObjects;
                    resolve(this._DebitTransactions);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            else {
                resolve(this._DebitTransactions);
            }
        });
    }
    get CreditTransactions() {
        return new Promise((resolve, reject) => {
            if (this.JournalEntryId !== 'New') {
                JournalEntry._LedgerTransactionRepository
                    .findAll({
                    where: {
                        TransactionType: enum_1.TransactionTypeOptions.CREDIT,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((creditTransaction) => {
                    const creditTransactionObjects = creditTransaction.map((creditTransactionData) => new ledger_transaction_1.default(this._DbTransaction, creditTransactionData.TransactionId));
                    return Promise.all(creditTransactionObjects);
                })
                    .then((creditTransactionObjects) => {
                    this._CreditTransactions = creditTransactionObjects;
                    resolve(this._CreditTransactions);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            else {
                resolve(this._CreditTransactions);
            }
        });
    }
    init(params) {
        if (params.JournalEntryId)
            this.JournalEntryId = params.JournalEntryId;
        if (params.CompanyId)
            this.CompanyId = params.CompanyId;
        if (params.Date)
            this.Date = params.Date;
        if (params.Name)
            this.Name = params.Name;
        if (params.Description)
            this.Description = params.Description;
        if (params.PostedById)
            this.PostedById = params.PostedById;
        if (params.PostedToAccSystemYN)
            this.PostedToAccSystemYN = params.PostedToAccSystemYN;
        if (params.PostedDateTime)
            this.PostedDateTime = params.PostedDateTime;
    }
    getData() {
        return {
            JournalEntryId: this.JournalEntryId,
            Date: this.Date,
            Name: this.Name,
            Description: this.Description,
            PostedById: this.PostedById,
            PostedToAccSystemYN: this.PostedToAccSystemYN,
            PostedDateTime: this.PostedDateTime,
        };
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.RepositoryBase.create(this.getData);
        });
    }
    save(userId, dbTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.JournalEntryId = cuid();
                const data = yield this.RepositoryBase.create({
                    JournalEntryId: this.JournalEntryId,
                    CompanyId: this.CompanyId,
                    Date: this.Date,
                    Name: this.Name,
                    Description: this.Description,
                    PostedById: userId,
                    PostedToAccSystemYN: this.PostedToAccSystemYN,
                    PostedDateTime: this.PostedDateTime,
                }, { transaction: dbTransaction });
                return data;
            }
            catch (error) {
                throw error;
            }
        });
    }
    newLedgerTransaction(transactionType) {
        return __awaiter(this, void 0, void 0, function* () {
            const ledgerTransaction = new ledger_transaction_1.default(this._DbTransaction);
            ledgerTransaction.JournalEntryId = this.JournalEntryId;
            if (transactionType === enum_1.TransactionTypeOptions.DEBIT) {
                this._DebitTransactions.push(ledgerTransaction);
            }
            else if (transactionType === enum_1.TransactionTypeOptions.CREDIT) {
                this._CreditTransactions.push(ledgerTransaction);
            }
            return ledgerTransaction;
        });
    }
}
exports.default = JournalEntry;
JournalEntry._RepositoryBase = new journal_entry_repository_1.JournalEntryRepository();
JournalEntry._LedgerTransactionRepository = new ledger_transaction_repository_1.LedgerTransactionRepository();
//# sourceMappingURL=journal-entry.js.map