"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cuid = require("cuid");
const transaction_type_enum_1 = require("../enum/transaction-type.enum");
const ledger_transaction_repository_1 = require("./ledger-transaction.repository");
class LedgerTransaction {
    constructor(dbTransaction, transactionId) {
        this.TransactionId = 'New';
        this.TransactionType = transaction_type_enum_1.TransactionTypeOptions.DEBIT;
        this.JournalEntryId = '';
        this.AccountNo = '';
        this.Date = new Date();
        this.Name = '';
        this.Description = '';
        this.Currency = 'MYR';
        this.DebitAmount = 0;
        this.CreditAmount = 0;
        this.RelatedObjectId = '';
        this.RelatedObjectType = '';
        this.RelatedDocNo = '';
        this.RelatedPaymentId = '';
        if (dbTransaction) {
            this._DbTransaction = dbTransaction;
        }
        if (transactionId) {
            this.TransactionId = transactionId;
        }
    }
    init(params) {
        if (params.TransactionId)
            this.TransactionId = params.TransactionId;
        if (params.TransactionType)
            this.TransactionType = params.TransactionType;
        if (params.JournalEntryId)
            this.JournalEntryId = params.JournalEntryId;
        if (params.AccountNo)
            this.AccountNo = params.AccountNo;
        if (params.Date)
            this.Date = params.Date;
        if (params.Description)
            this.Description = params.Description;
        if (params.Currency)
            this.Currency = params.Currency;
        if (params.DebitAmount)
            this.DebitAmount = params.DebitAmount;
        if (params.CreditAmount)
            this.CreditAmount = params.CreditAmount;
        if (params.RelatedObjectId)
            this.RelatedObjectId = params.RelatedObjectId;
        if (params.RelatedObjectType)
            this.RelatedObjectType = params.RelatedObjectType;
        if (params.Name)
            this.Name = params.Name;
    }
    getData() {
        return {
            TransactionId: this.TransactionId,
            TransactionType: this.TransactionType,
            JournalEntryId: this.JournalEntryId,
            AccountNo: this.AccountNo,
            Date: this.Date,
            Name: this.Name,
            Description: this.Description,
            Currency: this.Currency,
            DebitAmount: this.DebitAmount,
            CreditAmount: this.CreditAmount,
            RelatedObjectId: this.RelatedObjectId,
            RelatedObjectType: this.RelatedObjectType,
            RelatedDocNo: this.RelatedDocNo,
            RelatedPaymentId: this.RelatedPaymentId,
        };
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield LedgerTransaction._LedgerTransactionRepository.create(this.getData, { transaction: this._DbTransaction });
        });
    }
    findAll(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield LedgerTransaction._LedgerTransactionRepository.findAll(options);
        });
    }
    findAllWithPagination(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield LedgerTransaction._LedgerTransactionRepository.findAllWithPagination(options);
        });
    }
    accountTransactionHistory(accountNo) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.findAll({
                where: {
                    AccountNo: accountNo,
                },
                order: [['Date', 'DESC']],
            });
        });
    }
    newLedgerTransaction(transactionType, journalEntryId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.init({
                TransactionId: cuid(),
                TransactionType: transactionType,
                JournalEntryId: journalEntryId,
                Name: '',
                AccountNo: '',
                Date: new Date(),
                Description: '',
                Currency: 'MYR',
                DebitAmount: 0,
                CreditAmount: 0,
                RelatedObjectId: '',
                RelatedObjectType: '',
                RelatedDocNo: '',
                RelatedPaymentId: '',
            });
            if (transactionType === transaction_type_enum_1.TransactionTypeOptions.DEBIT) {
                this.TransactionType = transaction_type_enum_1.TransactionTypeOptions.DEBIT;
            }
            else if (transactionType === transaction_type_enum_1.TransactionTypeOptions.CREDIT) {
                this.TransactionType = transaction_type_enum_1.TransactionTypeOptions.CREDIT;
            }
            if (journalEntryId) {
                this.JournalEntryId = journalEntryId;
            }
            return yield this.create();
        });
    }
}
exports.default = LedgerTransaction;
LedgerTransaction._LedgerTransactionRepository = new ledger_transaction_repository_1.LedgerTransactionRepository();
//# sourceMappingURL=ledger-transaction.js.map