"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("@tomei/general");
const payment_method_repository_1 = require("./payment-method.repository");
const payment_method_type_1 = require("../payment-method-type/payment-method-type");
const payment_method_type_repository_1 = require("../payment-method-type/payment-method-type.repository");
class PaymentMethod extends general_1.ObjectBase {
    constructor(dbTransaction, methodId) {
        super();
        this.MethodId = 'New';
        this.Name = '';
        this.CompanyId = '';
        this._PaymentMethodTypes = [];
        if (dbTransaction) {
            this._DbTransaction = dbTransaction;
        }
        if (methodId) {
            this.MethodId = methodId;
            PaymentMethod._RepositoryBase
                .findOne({
                where: {
                    MethodId: methodId,
                },
                transaction: dbTransaction,
            })
                .then((methodData) => {
                if (methodData) {
                    this.Name = methodData.Name;
                }
                else {
                    throw new general_1.RecordNotFoundError('No record found.');
                }
            })
                .catch((err) => {
                console.log('Payment method constructor err: ', err);
            });
        }
    }
    get ObjectId() {
        return this.MethodId;
    }
    get ObjectName() {
        return this.Name;
    }
    get TableName() {
        return 'finance_PaymentMethod';
    }
    get PaymentMethodTypes() {
        return new Promise((resolve, reject) => {
            if (this.MethodId !== 'New') {
                PaymentMethod._PaymentMethodTypeRepository
                    .findAll({
                    where: {
                        MethodId: this.MethodId,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((paymentMethodTypes) => {
                    const paymentMethodTypeObjects = paymentMethodTypes.map((paymentMethodTypeData) => new payment_method_type_1.default(this._DbTransaction, paymentMethodTypeData.MethodTypeId));
                    return Promise.all(paymentMethodTypeObjects);
                })
                    .then((paymentMethodTypeObjects) => {
                    this._PaymentMethodTypes = paymentMethodTypeObjects;
                    resolve(this._PaymentMethodTypes);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            else {
                resolve(this._PaymentMethodTypes);
            }
        });
    }
    newPaymentMethodType(payload) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const newPaymentMethodType = yield PaymentMethod._PaymentMethodTypeRepository.create(payload);
                this._PaymentMethodTypes.push(newPaymentMethodType);
            }
            catch (err) {
                throw err;
            }
        });
    }
}
exports.default = PaymentMethod;
PaymentMethod._RepositoryBase = new payment_method_repository_1.PaymentMethodRepository();
PaymentMethod._PaymentMethodTypeRepository = new payment_method_type_repository_1.PaymentMethodTypeRepository();
//# sourceMappingURL=payment-method.js.map