"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const general_1 = require("@tomei/general");
const account_system_entity_1 = require("../account-system-entity/account-system-entity");
const payment_repository_1 = require("./payment.repository");
const payment_item_repository_1 = require("../payment-item/payment-item.repository");
const payment_item_1 = require("../payment-item/payment-item");
const payment_paid_with_1 = require("../payment-paid-with/payment-paid-with");
const payment_paid_with_repository_1 = require("../payment-paid-with/payment-paid-with.repository");
class Payment extends account_system_entity_1.AccountSystemEntity {
    get PaymentId() {
        return this._PaymentId;
    }
    set PaymentId(id) {
        this._PaymentId = id;
    }
    get RepositoryBase() {
        return Payment._RepositoryBase;
    }
    get ObjectId() {
        return this.PaymentId;
    }
    get ObjectName() {
        return this.PaymentId;
    }
    get TableName() {
        return 'finance_Payment';
    }
    constructor(dbTransaction, paymentId) {
        super();
        this._PaymentId = 'New';
        this.Description = '';
        this.Currency = 'MYR';
        this.Amount = 0;
        this.ReceivedBy = '';
        this.UpdatedBy = '';
        this.Remarks = '';
        this._PaymentItems = [];
        this._PaymentPaidWith = [];
        if (dbTransaction) {
            this._DbTransaction = dbTransaction;
        }
        if (paymentId) {
            this.RepositoryBase.findOne({
                where: {
                    PaymentId: paymentId,
                },
                transaction: dbTransaction,
            })
                .then((paymentData) => {
                if (paymentData) {
                    this.PaymentType = paymentData.PaymentType;
                    this.PaymentDate = paymentData.PaymentDate;
                    this.Description = paymentData.Description;
                    this.Currency = paymentData.Currency;
                    this.Amount = paymentData.Amount;
                    this.Status = paymentData.Status;
                    this.ReceivedBy = paymentData.ReceivedBy;
                    this.UpdatedAt = paymentData.UpdatedAt;
                    this.UpdatedBy = paymentData.UpdatedBy;
                    this.Remarks = paymentData.Remarks;
                    this.AccSystemRefId = paymentData.AccSystemRefId;
                    this.PostedToAccSystemYN = paymentData.PostedToAccSystemYN;
                    this.PostedById = paymentData.PostedById;
                    this.PostedDateTime = paymentData.PostedDateTime;
                }
                else {
                    const notFoundError = new general_1.RecordNotFoundError('No Record Found.');
                    throw notFoundError;
                }
            })
                .catch((err) => {
                console.log('Payment Class constructor err: ', err);
            });
        }
    }
    get PaymentItems() {
        return new Promise((resolve, reject) => {
            if (this.PaymentId !== 'New') {
                Payment._PaymentItemRepository
                    .findAll({
                    where: {
                        PaymentId: this.PaymentId,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((paymentItems) => {
                    const paymentItemObjects = paymentItems.map((paymentItem) => {
                        new payment_item_1.default(new Payment(this._DbTransaction, paymentItem.PaymentId), {
                            ObjectId: this.ObjectId,
                            ObjectName: this.ObjectName,
                            TableName: this.TableName,
                        });
                    });
                    return Promise.all(paymentItemObjects);
                })
                    .then((paymentItemObjects) => {
                    this._PaymentItems = paymentItemObjects;
                    resolve(this._PaymentItems);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            resolve(this._PaymentItems);
        });
    }
    get PaymentPaidWith() {
        return new Promise((resolve, reject) => {
            if (this.PaymentId !== 'New') {
                Payment._PaymentPaidWithRepository
                    .findAll({
                    where: {
                        PaymentId: this.PaymentId,
                    },
                    transaction: this._DbTransaction,
                })
                    .then((paymentPaidWithItems) => {
                    const paymentPaidWithObjects = paymentPaidWithItems.map((paymentPaidWithItem) => {
                        new payment_paid_with_1.default(new Payment(this._DbTransaction, paymentPaidWithItem.PaymentId), paymentPaidWithItem.MethodTypeId);
                    });
                    return Promise.all(paymentPaidWithObjects);
                })
                    .then((paymentPaidWithObjects) => {
                    this._PaymentPaidWith = paymentPaidWithObjects;
                    resolve(this._PaymentPaidWith);
                })
                    .catch((err) => {
                    reject(err);
                });
            }
            resolve(this._PaymentPaidWith);
        });
    }
    newPaymentItem(objectBeingPaidFor) {
        const paymentItem = new payment_item_1.default(this, objectBeingPaidFor);
        this._PaymentItems.push(paymentItem);
        return paymentItem;
    }
    newPaymentPaidWith(paymentMethodTypeId) {
        const paymentPaidWith = new payment_paid_with_1.default(this, paymentMethodTypeId);
        this._PaymentPaidWith.push(paymentPaidWith);
        return paymentPaidWith;
    }
    init(Params) {
        if (Params.Currency)
            this.Currency = Params.Currency;
        if (Params.PaymentType)
            this.PaymentType = Params.PaymentType;
    }
}
exports.default = Payment;
Payment._RepositoryBase = new payment_repository_1.PaymentRepository();
Payment._PaymentItemRepository = new payment_item_repository_1.PaymentItemRepository();
Payment._PaymentPaidWithRepository = new payment_paid_with_repository_1.PaymentPaidWithRepository();
//# sourceMappingURL=payment.js.map