import * as cuid from 'cuid';
import { IJournalEntryAttr } from './interfaces/journal-entry-attr.interface';
import { TransactionTypeOptions } from '../enum';
import FinanceCompany from '../finance-company/finance-company';
import JournalEntryModel from '../models/journal-entry.entity';
import { AccountSystemEntity } from '../account-system-entity/account-system-entity';
import { LedgerTransactionRepository } from '../ledger-transaction/ledger-transaction.repository';
import LedgerTransaction from '../ledger-transaction/ledger-transaction';
import { JournalEntryRepository } from './journal-entry.repository';
import { RecordNotFoundError } from '@tomei/general';

export default class JournalEntry extends AccountSystemEntity {
  private _JournalEntryId = 'New';
  Date = new Date();
  Name = '';
  Description = '';

  private static _RepositoryBase = new JournalEntryRepository();
  private static _LedgerTransactionRepository =
    new LedgerTransactionRepository();

  private _DbTransaction: any;
  private _DebitTransactions: LedgerTransaction[] = [];
  private _CreditTransactions: LedgerTransaction[] = [];

  get JournalEntryId() {
    return this._JournalEntryId;
  }

  private set JournalEntryId(id: string) {
    this._JournalEntryId = id;
  }

  get RepositoryBase() {
    return JournalEntry._RepositoryBase;
  }

  get TableName() {
    return 'finance_JournalEntry';
  }

  get ObjectName() {
    return this.Name;
  }

  get ObjectId() {
    return this._JournalEntryId;
  }

  constructor(dbTransaction?: any, journalEntryId?: string) {
    super();
    if (dbTransaction) {
      this._DbTransaction = dbTransaction;
    }
    if (journalEntryId) {
      this.RepositoryBase.findOne({
        where: {
          JournalEntryId: journalEntryId,
        },
        transaction: dbTransaction,
      })
        .then((journalEntryData) => {
          if (journalEntryData) {
            this.CompanyId = journalEntryData.CompanyId;
            this.Date = journalEntryData.Date;
            this.Name = journalEntryData.Name;
            this.Description = journalEntryData.Description;
            this.AccSystemRefId = journalEntryData.AccSystemRefId;
            this.PostedToAccSystemYN = journalEntryData.PostedToAccSystemYN;
            this.PostedById = journalEntryData.PostedById;
            this.PostedDateTime = journalEntryData.PostedDateTime;
          } else {
            const notFoundError = new RecordNotFoundError('No Record Found.');
            throw notFoundError;
          }
        })
        .catch((err) => {
          console.log('Journal entry constructor err: ', err);
        });
    } else {
      this.init({
        JournalEntryId: 'New',
        CompanyId: '',
        Date: new Date(),
        Name: '',
        Description: '',
        AccSystemRefId: '',
        PostedToAccSystemYN: 'N',
        PostedById: '',
        PostedDateTime: null,
      });
    }
  }

  get DebitTransactions(): Promise<LedgerTransaction[]> {
    return new Promise((resolve, reject) => {
      if (this.JournalEntryId !== 'New') {
        JournalEntry._LedgerTransactionRepository
          .findAll({
            where: {
              TransactionType: TransactionTypeOptions.DEBIT,
            },
            transaction: this._DbTransaction,
          })
          .then((debitTransactions) => {
            const debitTransactionObjects = debitTransactions.map(
              (debitTransactionData) =>
                new LedgerTransaction(
                  this._DbTransaction,
                  debitTransactionData.TransactionId,
                ),
            );
            return Promise.all(debitTransactionObjects);
          })
          .then((debitTransactionObjects) => {
            this._DebitTransactions = debitTransactionObjects;
            resolve(this._DebitTransactions);
          })
          .catch((err) => {
            reject(err);
          });
      } else {
        resolve(this._DebitTransactions);
      }
    });
  }

  get CreditTransactions(): Promise<LedgerTransaction[]> {
    return new Promise((resolve, reject) => {
      if (this.JournalEntryId !== 'New') {
        JournalEntry._LedgerTransactionRepository
          .findAll({
            where: {
              TransactionType: TransactionTypeOptions.CREDIT,
            },
            transaction: this._DbTransaction,
          })
          .then((creditTransaction) => {
            const creditTransactionObjects = creditTransaction.map(
              (creditTransactionData) =>
                new LedgerTransaction(
                  this._DbTransaction,
                  creditTransactionData.TransactionId,
                ),
            );
            return Promise.all(creditTransactionObjects);
          })
          .then((creditTransactionObjects) => {
            this._CreditTransactions = creditTransactionObjects;
            resolve(this._CreditTransactions);
          })
          .catch((err) => {
            reject(err);
          });
      } else {
        resolve(this._CreditTransactions);
      }
    });
  }

  // public get DebitTransactions() {
  //   if (this._DebitTransactions === null) {
  //     // this._DebitTransactions = new LedgerTransaction(
  //     //   this.ledgerTransactionRepository,
  //     // );
  //     if (this.JournalEntryId !== 'New') {
  //       JournalEntry._LedgerTransactionRepository
  //         .findAll({
  //           where: {
  //             JournalEntryId: {
  //               [Op.ne]: 'New',
  //             },
  //             TransactionType: TransactionTypeOptions.DEBIT,
  //           },
  //           transaction: this._DbTransaction,
  //         })
  //         .then((allDebitTransactions) => {
  //           this._DebitTransactions = allDebitTransactions;
  //         });
  //     }
  //   }

  //   return this._DebitTransactions;
  // }

  // public get CreditTransactions() {
  //   if (this._CreditTransactions === null) {
  //     // this._CreditTransactions = new LedgerTransaction(
  //     //   this.ledgerTransactionRepository,
  //     // );
  //     if (this.JournalEntryId !== 'New') {
  //       JournalEntry._LedgerTransactionRepository
  //         .findAll({
  //           where: {
  //             JournalEntryId: {
  //               [Op.ne]: 'New',
  //             },
  //             TransactionType: TransactionTypeOptions.CREDIT,
  //           },
  //           transaction: this._DbTransaction,
  //         })
  //         .then((allCreditTransactions) => {
  //           this._CreditTransactions = allCreditTransactions;
  //         });
  //     }
  //   }

  //   return this._CreditTransactions;
  // }

  init(params: IJournalEntryAttr) {
    if (params.JournalEntryId) this.JournalEntryId = params.JournalEntryId;
    if (params.CompanyId) this.CompanyId = params.CompanyId;
    if (params.Date) this.Date = params.Date;
    if (params.Name) this.Name = params.Name;
    if (params.Description) this.Description = params.Description;
    if (params.PostedById) this.PostedById = params.PostedById;
    if (params.PostedToAccSystemYN)
      this.PostedToAccSystemYN = params.PostedToAccSystemYN;
    if (params.PostedDateTime) this.PostedDateTime = params.PostedDateTime;
  }

  getData() {
    return {
      JournalEntryId: this.JournalEntryId,
      Date: this.Date,
      Name: this.Name,
      Description: this.Description,
      PostedById: this.PostedById,
      PostedToAccSystemYN: this.PostedToAccSystemYN,
      PostedDateTime: this.PostedDateTime,
    };
  }

  async create() {
    return await this.RepositoryBase.create(this.getData);
  }

  async save(userId: string, dbTransaction?: any): Promise<JournalEntryModel> {
    try {
      this.JournalEntryId = cuid();
      const data = await this.RepositoryBase.create(
        {
          JournalEntryId: this.JournalEntryId,
          CompanyId: this.CompanyId,
          Date: this.Date,
          Name: this.Name,
          Description: this.Description,
          PostedById: userId,
          PostedToAccSystemYN: this.PostedToAccSystemYN,
          PostedDateTime: this.PostedDateTime,
        },
        { transaction: dbTransaction },
      );
      return data;
    } catch (error) {
      throw error;
    }
  }

  /**
   * Create a new LedgerTransaction
   *
   * @see {@link FinanceCompany.issueInvoice} - for implementation
   *
   * @param transactionType - Indicating the type of ledger transaction to create
   *
   * @returns {LedgerTransaction}
   */
  async newLedgerTransaction(
    transactionType: TransactionTypeOptions,
  ): Promise<LedgerTransaction> {
    const ledgerTransaction = new LedgerTransaction(this._DbTransaction);
    ledgerTransaction.JournalEntryId = this.JournalEntryId;

    if (transactionType === TransactionTypeOptions.DEBIT) {
      this._DebitTransactions.push(ledgerTransaction);
    } else if (transactionType === TransactionTypeOptions.CREDIT) {
      this._CreditTransactions.push(ledgerTransaction);
    }

    return ledgerTransaction;
  }
}
