import * as cuid from 'cuid';
import { ILedgerTransactionAttr } from './interfaces/ledger-transaction-attr.interface';
import { TransactionTypeOptions } from '../enum/transaction-type.enum';
import { LedgerTransactionRepository } from './ledger-transaction.repository';

export default class LedgerTransaction {
  TransactionId = 'New';
  TransactionType = TransactionTypeOptions.DEBIT;
  JournalEntryId = '';
  AccountNo = '';
  Date = new Date();
  Name = '';
  Description = '';
  Currency = 'MYR';
  DebitAmount = 0;
  CreditAmount = 0;
  RelatedObjectId = '';
  RelatedObjectType = '';
  RelatedDocNo = '';
  RelatedPaymentId = '';

  private _DbTransaction: any;

  private static _LedgerTransactionRepository =
    new LedgerTransactionRepository();

  constructor(dbTransaction?: any, transactionId?: string) {
    if (dbTransaction) {
      this._DbTransaction = dbTransaction;
    }
    if (transactionId) {
      this.TransactionId = transactionId;
    }
  }

  init(params?: ILedgerTransactionAttr) {
    if (params.TransactionId) this.TransactionId = params.TransactionId;
    if (params.TransactionType) this.TransactionType = params.TransactionType;
    if (params.JournalEntryId) this.JournalEntryId = params.JournalEntryId;
    if (params.AccountNo) this.AccountNo = params.AccountNo;
    if (params.Date) this.Date = params.Date;
    if (params.Description) this.Description = params.Description;
    if (params.Currency) this.Currency = params.Currency;
    if (params.DebitAmount) this.DebitAmount = params.DebitAmount;
    if (params.CreditAmount) this.CreditAmount = params.CreditAmount;
    if (params.RelatedObjectId) this.RelatedObjectId = params.RelatedObjectId;
    if (params.RelatedObjectType)
      this.RelatedObjectType = params.RelatedObjectType;
    if (params.Name) this.Name = params.Name;
  }

  getData() {
    return {
      TransactionId: this.TransactionId,
      TransactionType: this.TransactionType,
      JournalEntryId: this.JournalEntryId,
      AccountNo: this.AccountNo,
      Date: this.Date,
      Name: this.Name,
      Description: this.Description,
      Currency: this.Currency,
      DebitAmount: this.DebitAmount,
      CreditAmount: this.CreditAmount,
      RelatedObjectId: this.RelatedObjectId,
      RelatedObjectType: this.RelatedObjectType,
      RelatedDocNo: this.RelatedDocNo,
      RelatedPaymentId: this.RelatedPaymentId,
    };
  }

  async create() {
    return await LedgerTransaction._LedgerTransactionRepository.create(
      this.getData,
      { transaction: this._DbTransaction },
    );
  }

  async findAll(options: any) {
    return await LedgerTransaction._LedgerTransactionRepository.findAll(
      options,
    );
  }

  async findAllWithPagination(options: any) {
    return await LedgerTransaction._LedgerTransactionRepository.findAllWithPagination(
      options,
    );
  }

  async accountTransactionHistory(accountNo: string) {
    return await this.findAll({
      where: {
        AccountNo: accountNo,
      },
      order: [['Date', 'DESC']],
    });
  }

  async newLedgerTransaction(
    transactionType: TransactionTypeOptions,
    journalEntryId?: string,
  ): Promise<any> {
    this.init({
      TransactionId: cuid(),
      TransactionType: transactionType,
      JournalEntryId: journalEntryId,
      //TODO: AccountNo should be Filled In. Not sure with What
      Name: '',
      AccountNo: '',
      Date: new Date(),
      Description: '',
      Currency: 'MYR',
      DebitAmount: 0,
      CreditAmount: 0,
      RelatedObjectId: '',
      RelatedObjectType: '',
      RelatedDocNo: '',
      RelatedPaymentId: '',
    });

    if (transactionType === TransactionTypeOptions.DEBIT) {
      this.TransactionType = TransactionTypeOptions.DEBIT;
    } else if (transactionType === TransactionTypeOptions.CREDIT) {
      this.TransactionType = TransactionTypeOptions.CREDIT;
    }

    if (journalEntryId) {
      this.JournalEntryId = journalEntryId;
    }

    return await this.create();
  }
}
