import { ApiProperty } from '@nestjs/swagger';
import {
  ForeignKey,
  BelongsTo,
  Column,
  CreatedAt,
  Model,
  Table,
  DataType,
  UpdatedAt,
  HasMany,
} from 'sequelize-typescript';
import LedgerTransactionModel from './ledger-transaction.entity';
import DocumentItemModel from './document-item.entity';
import FinanceCompanyModel from './finance-company.entity';
import PaymentMethodTypeModel from './payment-method-type.entity';

@Table({ tableName: 'finance_Account', createdAt: false, updatedAt: false })
export default class AccountModel extends Model {
  @ApiProperty({ type: String, description: 'AccountNo' })
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  AccountNo: string;

  @ApiProperty({
    example: 'cl6nzvo780000qcw38ihpd8w6',
    description: 'CompanyId',
  })
  @ForeignKey(() => FinanceCompanyModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CompanyId: string;

  //copy from finance account migration table
  @ApiProperty({ type: String, description: 'Parent account of this object' })
  @Column({
    type: DataType.STRING(30),
    allowNull: true,
  })
  ParentAccountNo: string;

  @ApiProperty({
    type: String,
    description: 'Name of the account',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(200),
  })
  Name: string;

  @ApiProperty({
    type: String,
    description: 'Account description',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(2000),
  })
  Description: string;

  @ApiProperty({
    type: String,
    description: 'Account Type',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(100),
  })
  AccountType: string;

  @ApiProperty({
    type: String,
    description: 'Account Sub Type',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(100),
  })
  AccountSubType: string;

  @ApiProperty({
    type: String,
    description: 'Owner id of the account',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  OwnerId: string;

  @ApiProperty({
    type: String,
    description: 'Owner type',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(200),
  })
  OwnerType: string;

  @ApiProperty({ type: String, description: 'Associated Object id' })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  RelatedObjectId: string;

  @ApiProperty({ type: String, description: 'Associated Object type' })
  @Column({
    allowNull: true,
    type: DataType.STRING(200),
  })
  RelatedObjectType: string;

  @ApiProperty({
    example: new Date(),
    description: 'Timestamp for data creation.',
  })
  @CreatedAt
  CreatedAt: Date;

  @ApiProperty({
    example: '138140891dd211b288d34bc7b4312a49',
    description: 'The CreatedById for Media.',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CreatedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'Timestamp for latest data modification',
  })
  @UpdatedAt
  UpdatedAt: Date;

  @ApiProperty({
    example: '138140891dd211b288d34bc7b4312a49',
    description: 'The UpdatedById for Media.',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  UpdatedById: string;

  @ApiProperty({
    type: String,
    description: 'Account id in finance system',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(10),
  })
  AccSystemRefId: string;

  @ApiProperty({
    example: 'N',
    description: 'PostedToAccSystemYN',
  })
  @Column({
    allowNull: false,
    type: DataType.ENUM('Y', 'N'),
  })
  PostedToAccSystemYN: string;

  @ApiProperty({
    example: '138140891dd211b288',
    description: 'PostedById',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  PostedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'PostedToAccSystem Date',
  })
  @Column({
    allowNull: true,
    type: DataType.DATE,
  })
  PostedDateTime: Date;

  @HasMany(() => PaymentMethodTypeModel)
  PaymentMethodTypes: PaymentMethodTypeModel[];

  @HasMany(() => DocumentItemModel)
  DocumentItems: DocumentItemModel[];

  @HasMany(() => LedgerTransactionModel)
  LedgerTransactions: LedgerTransactionModel[];

  @BelongsTo(() => FinanceCompanyModel)
  FinanceCompany: FinanceCompanyModel;
}
