import { ApiProperty } from '@nestjs/swagger';
import {
  ForeignKey,
  BelongsTo,
  Column,
  DataType,
  Model,
  Table,
} from 'sequelize-typescript';
import FinanceCompanyModel from './finance-company.entity';

@Table({ tableName: 'finance_Customer', createdAt: false, updatedAt: false })
export default class FinanceCustomerModel extends Model {
  @ApiProperty({ type: String, description: 'CustomerId' })
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  CustomerId: string;

  @ApiProperty({ type: String, description: 'CompanyId' })
  @ForeignKey(() => FinanceCompanyModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CompanyId: string;

  @ApiProperty({
    type: String,
    description: 'Cust System Code eg. "EZC", "CRM"',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(10),
  })
  CustSystemCode: string;

  @ApiProperty({
    type: String,
    description: 'CustSystemRefId',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CustSystemRefId: string;

  @ApiProperty({
    type: String,
    description: 'Account Customer Id in API',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  AccSystemRefId: string;

  @ApiProperty({
    type: String,
    description: 'PostedToAccSystemYN',
    example: 'Y',
  })
  @Column({
    allowNull: false,
    type: DataType.CHAR(1),
  })
  PostedToAccSystemYN: string;

  @ApiProperty({
    example: '138140891dd211b288d34bc7b4312a49',
    description: 'PostedById',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  PostedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'PostedToAccSystem Date',
  })
  @Column({
    allowNull: true,
    type: DataType.DATE,
  })
  PostedDateTime: Date;

  @BelongsTo(() => FinanceCompanyModel)
  FinanceCompany: FinanceCompanyModel;
}
