import {
  BelongsTo,
  Column,
  DataType,
  ForeignKey,
  Model,
  Table,
} from 'sequelize-typescript';
import DocumentModel from './document.entity';
import AccountModel from './account.entity';

@Table({ tableName: 'finance_DocumentItem', timestamps: false })
export default class DocumentItemModel extends Model {
  @Column({
    type: DataType.STRING(30),
    allowNull: false,
    primaryKey: true,
  })
  DocumentItemId: string;

  @ForeignKey(() => DocumentModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  DocNo: string;

  @Column({
    type: DataType.STRING(200),
    allowNull: false,
  })
  Name: string;

  @Column({
    type: DataType.STRING(200),
    allowNull: false,
  })
  NameBM: string;

  @Column({
    type: DataType.STRING(1000),
    allowNull: true,
  })
  Description: string;

  @Column({
    type: DataType.STRING(30),
    allowNull: true,
  })
  ItemId: string;

  @Column({
    type: DataType.STRING(200),
    allowNull: true,
  })
  ItemType: string;

  @Column({
    type: DataType.STRING(30),
    allowNull: true,
  })
  ItemSKU: string;

  @Column({
    type: DataType.STRING(30),
    allowNull: true,
  })
  ItemSerialNo: string;

  @Column({
    type: DataType.CHAR(3),
    allowNull: false,
  })
  Currency: string;

  @Column({
    type: DataType.DECIMAL(10, 2),
    allowNull: true,
  })
  UnitPrice: number;

  @Column({
    type: DataType.DECIMAL(10, 2),
    allowNull: true,
  })
  Quantity: number;

  @Column({
    type: DataType.STRING(20),
    allowNull: true,
  })
  QuantityUOM: string;

  @Column({
    type: DataType.DECIMAL(10, 2),
    allowNull: false,
  })
  Amount: number;

  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  TaxCode: string;

  @Column({
    type: DataType.DECIMAL(10, 2),
    allowNull: true,
  })
  TaxAmount: number;

  @Column({
    type: DataType.DECIMAL(10, 2),
    allowNull: true,
  })
  TaxRate: number;

  @Column({
    allowNull: true,
    type: DataType.ENUM('Y', 'N'),
  })
  TaxInclusiveYN: string;

  @ForeignKey(() => AccountModel)
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  DtAccountNo: string;

  @ForeignKey(() => AccountModel)
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  CtAccountNo: string;

  @BelongsTo(() => DocumentModel)
  Document: DocumentModel;

  @BelongsTo(() => AccountModel)
  Account: AccountModel;
}
