import { ApiProperty } from '@nestjs/swagger';
import {
  ForeignKey,
  BelongsTo,
  Column,
  DataType,
  HasMany,
  Model,
  Table,
  CreatedAt,
  UpdatedAt,
} from 'sequelize-typescript';
import { DocType, DocumentStatus } from '../enum';
import FinanceCompanyModel from './finance-company.entity';
import DocumentItemModel from './document-item.entity';

@Table({
  tableName: 'finance_Document',
  // timestamps: false,
})
export default class DocumentModel extends Model {
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  DocNo: string;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  DocType: DocType;

  @Column({
    allowNull: false,
    type: DataType.DATE,
  })
  DocDate: Date;

  @ApiProperty({
    example: 'cl6nzvo780000qcw38ihpd8w6',
    description: 'CompanyId',
  })
  @ForeignKey(() => FinanceCompanyModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CompanyId: string;

  @Column({
    allowNull: false,
    type: DataType.CHAR(3),
  })
  Currency: string;

  @Column({
    allowNull: false,
    type: DataType.DECIMAL(10, 2),
  })
  Amount: number;

  @Column({
    allowNull: false,
    type: DataType.STRING(1000),
  })
  Description: string;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  Status: DocumentStatus;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  IssuedById: string;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  IssuedToId: string;

  @Column({
    allowNull: false,
    type: DataType.STRING(200),
  })
  IssuedToType: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  RelatedObjectId: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(200),
  })
  RelatedObjectType: string;

  @ApiProperty({
    example: '138140891dd211b288d34bc7b4312a49',
    description: 'CreatedById.',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CreatedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'Timestamp for data creation.',
  })
  @CreatedAt
  CreatedAt: Date;

  @ApiProperty({
    example: '138140891dd211b288d34bc7b4312a49',
    description: 'UpdatedById.',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  UpdatedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'Timestamp for latest data modification',
  })
  @UpdatedAt
  UpdatedAt: Date;

  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  DocPDFFileMediaId: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  DocHTMLFileMediaId: string;

  @ApiProperty({
    type: String,
    description: 'Account Customer Id in API',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  AccSystemRefId: string;

  @ApiProperty({
    example: 'N',
    description: 'PostedToAccSystemYN',
  })
  @Column({
    allowNull: false,
    type: DataType.ENUM('Y', 'N'),
  })
  PostedToAccSystemYN: string;

  @ApiProperty({
    example: '138140891dd211b288',
    description: 'PostedById',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  PostedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'PostedToAccSystem Date',
  })
  @Column({
    allowNull: true,
    type: DataType.DATE,
  })
  PostedDateTime: Date;

  @Column({
    allowNull: false,
    type: DataType.ENUM('Y', 'N'),
  })
  UseAccSystemDocYN: string;

  @HasMany(() => DocumentItemModel)
  DocumentItems: DocumentItemModel[];

  @BelongsTo(() => FinanceCompanyModel)
  FinanceCompany: FinanceCompanyModel;
}
