import { ApiProperty } from '@nestjs/swagger';
import {
  ForeignKey,
  BelongsTo,
  Column,
  Model,
  Table,
  DataType,
  HasMany,
} from 'sequelize-typescript';
import LedgerTransactionModel from './ledger-transaction.entity';
import FinanceCompanyModel from './finance-company.entity';

@Table({
  tableName: 'finance_JournalEntry',
  timestamps: false,
})
export default class JournalEntryModel extends Model {
  @ApiProperty({
    example: 'cl6nzvo780000qcw38ihpd8w6',
    description: 'CompanyId',
  })
  @ForeignKey(() => FinanceCompanyModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CompanyId: string;

  @ApiProperty({ type: String, description: 'JournalEntryId' })
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  JournalEntryId: string;

  @ApiProperty({ type: Date, description: 'Date' })
  @Column({
    allowNull: true,
    type: DataType.DATE,
  })
  Date: Date;

  @ApiProperty({
    type: String,
    description: 'Name of the journal entry',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(200),
  })
  Name: string;

  @ApiProperty({
    type: String,
    description: 'Description of the journal entry',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(1000),
  })
  Description: string;

  @ApiProperty({
    type: String,
    description: 'Account Customer Id in API',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  AccSystemRefId: string;

  @ApiProperty({
    example: 'N',
    description: 'PostedToAccSystemYN',
  })
  @Column({
    allowNull: false,
    type: DataType.ENUM('Y', 'N'),
  })
  PostedToAccSystemYN: string;

  @ApiProperty({
    example: '138140891dd211b288',
    description: 'PostedById',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  PostedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'PostedToAccSystem Date',
  })
  @Column({
    allowNull: true,
    type: DataType.DATE,
  })
  PostedDateTime: Date;

  @HasMany(() => LedgerTransactionModel)
  LedgerTransactions: LedgerTransactionModel[];

  @BelongsTo(() => FinanceCompanyModel)
  FinanceCompany: FinanceCompanyModel;
}
