import { ApiProperty } from '@nestjs/swagger';
import {
  Column,
  ForeignKey,
  Model,
  Table,
  DataType,
  BelongsTo,
} from 'sequelize-typescript';
import AccountModel from './account.entity';
import DocumentModel from './document.entity';
import JournalEntryModel from './journal-entry.entity';
import PaymentModel from './payment.entity';

@Table({
  tableName: 'finance_LedgerTransaction',
  timestamps: false,
})
export default class LedgerTransactionModel extends Model {
  @ApiProperty({ type: String, description: 'Transactionid' })
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  TransactionId: string;

  @ApiProperty({
    type: String,
    description: 'Transaction Type eg. "Credit", "Debit"',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(20),
  })
  TransactionType: string;

  @ApiProperty({ type: String, description: 'JournalEntryId' })
  @ForeignKey(() => JournalEntryModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  JournalEntryId: string;

  @ApiProperty({ type: String, description: 'AccountNo' })
  @ForeignKey(() => AccountModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  AccountNo: string;

  @ApiProperty({ type: Date, description: 'Date' })
  @Column({
    allowNull: false,
    type: DataType.DATE,
  })
  Date: Date;

  @ApiProperty({
    type: String,
    description: 'Description of the ledger transaction',
  })
  @Column({
    allowNull: false,
    type: DataType.STRING(1000),
  })
  Description: string;

  @ApiProperty({
    type: String,
    description: 'Currency',
  })
  @Column({
    allowNull: false,
    type: DataType.CHAR(3),
  })
  Currency: string;

  @ApiProperty({
    type: Number,
    description: 'Debit Amount of the ledger transaction',
  })
  @Column({
    allowNull: true,
    type: DataType.DECIMAL(10, 2),
  })
  DebitAmount: number;

  @ApiProperty({
    type: Number,
    description: 'Credit Amount of the ledger transaction',
  })
  @Column({
    allowNull: true,
    type: DataType.DECIMAL(10, 2),
  })
  CreditAmount: number;

  @ApiProperty({
    type: String,
    description: 'RelatedObjectId',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  RelatedObjectId: string;

  @ApiProperty({
    type: String,
    description: 'RelatedObjectType',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(200),
  })
  RelatedObjectType: string;

  @ApiProperty({ type: String, description: 'DocNo' })
  @ForeignKey(() => DocumentModel)
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  RelatedDocNo: string;

  @ApiProperty({ type: String, description: 'PaymentId' })
  @ForeignKey(() => PaymentModel)
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  RelatedPaymentId: string;

  @BelongsTo(() => AccountModel)
  Account: AccountModel;

  @BelongsTo(() => JournalEntryModel)
  JournalEntry: JournalEntryModel;

  @BelongsTo(() => DocumentModel)
  Document: DocumentModel;

  @BelongsTo(() => PaymentModel)
  Payment: PaymentModel;
}
