import { ApiProperty } from '@nestjs/swagger';
import {
  BelongsTo,
  Column,
  DataType,
  Model,
  Table,
  ForeignKey,
} from 'sequelize-typescript';
import AccountModel from './account.entity';
import PaymentMethodModel from './payment-method.entity';

@Table({
  tableName: 'finance_PaymentMethodType',
  createdAt: false,
  updatedAt: false,
})
export default class PaymentMethodTypeModel extends Model {
  @ForeignKey(() => PaymentMethodModel)
  @ApiProperty({ example: 'cl6nzvo780000qcw38ihpd8w6' })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  MethodId: string;

  @ApiProperty({ example: 'cl6nzvo780000qcw38ihpd8w6' })
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  MethodTypeId: string;

  @ApiProperty({ example: 'Cash' })
  @Column({
    allowNull: false,
    type: DataType.STRING(100),
  })
  Name: string;

  @ForeignKey(() => AccountModel)
  @ApiProperty({ example: 'cl6nzvo780000qcw38ihpd8w6' })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  AccountNo: string;

  @ApiProperty({ example: '100.0' })
  @Column({
    allowNull: false,
    type: DataType.DECIMAL(10, 2),
  })
  ProcessingFeeRate: number;

  @ForeignKey(() => AccountModel)
  @ApiProperty({ example: 'cl6nzvo780000qcw38ihpd8w6' })
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  ProcessingFeeAccountNo: string;

  @BelongsTo(() => AccountModel)
  Account: AccountModel;

  @BelongsTo(() => PaymentMethodModel)
  PaymentMethod: PaymentMethodModel;
}
