import { ApiProperty } from '@nestjs/swagger';
import {
  ForeignKey,
  BelongsTo,
  Column,
  DataType,
  HasMany,
  Model,
  Table,
} from 'sequelize-typescript';
import PaymentMethodTypeModel from './payment-method-type.entity';
import FinanceCompanyModel from './finance-company.entity';

@Table({
  tableName: 'finance_PaymentMethod',
  createdAt: false,
  updatedAt: false,
})
export default class PaymentMethodModel extends Model {
  @ApiProperty({ example: 'cl6nzvo780000qcw38ihpd8w6' })
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  MethodId: string;

  @ApiProperty({ example: 'cash' })
  @Column({
    allowNull: false,
    type: DataType.STRING(100),
  })
  Name: string;

  @ApiProperty({
    example: 'EZC',
    description: 'CompanyId',
  })
  @ForeignKey(() => FinanceCompanyModel)
  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  CompanyId: string;

  @HasMany(() => PaymentMethodTypeModel)
  PaymentMethodType: PaymentMethodTypeModel[];

  @BelongsTo(() => FinanceCompanyModel)
  FinanceCompany: FinanceCompanyModel;
}
