import { ApiProperty } from '@nestjs/swagger';
import {
  BelongsTo,
  Column,
  DataType,
  ForeignKey,
  Model,
  Table,
} from 'sequelize-typescript';
import { PaymentStatus } from '../enum';
import PaymentMethodTypeModel from './payment-method-type.entity';
import PaymentModel from './payment.entity';

@Table({
  tableName: 'finance_PaymentPaidWith',
  createdAt: false,
  updatedAt: false,
})
export default class PaymentPaidWithModel extends Model {
  @ApiProperty({
    example: 'cl6nzvo780000qcw38ihpd8w6',
    description: 'PaymentId',
  })
  @ForeignKey(() => PaymentModel)
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  PaymentId: string;

  @ApiProperty({
    example: 'cl6nzvo780000qcw38ihpd8w6',
    description: 'MethodTypeId',
  })
  @ForeignKey(() => PaymentMethodTypeModel)
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  MethodTypeId: string;

  @Column({
    allowNull: false,
    type: DataType.CHAR(3),
  })
  Currency: string;

  @Column({
    allowNull: false,
    type: DataType.DECIMAL(10, 2),
  })
  Amount: number;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  Status: PaymentStatus;

  @Column({
    allowNull: true,
    type: DataType.STRING(100),
  })
  TransactionId: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(200),
  })
  RefBank: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(300),
  })
  RefName: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(100),
  })
  RefNo: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(500),
  })
  RefOther1: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(500),
  })
  RefOther2: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(500),
  })
  RefOther3: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(500),
  })
  RefOther4: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(500),
  })
  RefOther5: string;

  @Column({
    allowNull: true,
    type: DataType.TEXT,
  })
  Remarks: string;

  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  PaymentMediaId: string;

  @BelongsTo(() => PaymentModel)
  Payment: PaymentModel;

  @BelongsTo(() => PaymentMethodTypeModel)
  PaymentMethodType: PaymentMethodTypeModel;
}
