import { ApiProperty } from '@nestjs/swagger';
import {
  Column,
  DataType,
  HasMany,
  Model,
  Table,
  UpdatedAt,
} from 'sequelize-typescript';
import { PaymentType, PaymentStatus } from '../enum';
import PaymentItemModel from './payment-item.entity';

@Table({
  tableName: 'finance_Payment',
  timestamps: true,
  createdAt: false,
})
export default class PaymentModel extends Model {
  @Column({
    primaryKey: true,
    allowNull: false,
    type: DataType.STRING(30),
  })
  PaymentId: string;

  @Column({
    allowNull: false,
    type: DataType.ENUM('Payment Received', 'Payout'),
  })
  PaymentType: PaymentType;

  @Column({
    allowNull: false,
    type: DataType.DATE,
  })
  PaymentDate: Date;

  @Column({
    allowNull: false,
    type: DataType.STRING(1000),
  })
  Description: string;

  @Column({
    allowNull: false,
    type: DataType.CHAR(3),
  })
  Currency: string;

  @Column({
    allowNull: false,
    type: DataType.DECIMAL(10, 2),
  })
  Amount: number;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  Status: PaymentStatus;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  ReceivedBy: string;

  @ApiProperty({
    example: new Date(),
    description: 'Timestamp for latest data modification',
  })
  @UpdatedAt
  UpdatedAt: Date;

  @Column({
    allowNull: false,
    type: DataType.STRING(30),
  })
  UpdatedBy: string;

  @Column({
    allowNull: true,
    type: DataType.TEXT,
  })
  Remarks: string;

  @ApiProperty({
    type: String,
    description: 'Account Customer Id in API',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  AccSystemRefId: string;

  @ApiProperty({
    example: 'N',
    description: 'PostedToAccSystemYN',
  })
  @Column({
    allowNull: false,
    type: DataType.ENUM('Y', 'N'),
  })
  PostedToAccSystemYN: string;

  @ApiProperty({
    example: '138140891dd211b288',
    description: 'PostedById',
  })
  @Column({
    allowNull: true,
    type: DataType.STRING(30),
  })
  PostedById: string;

  @ApiProperty({
    example: new Date(),
    description: 'PostedToAccSystem Date',
  })
  @Column({
    allowNull: true,
    type: DataType.DATE,
  })
  PostedDateTime: Date;

  @HasMany(() => PaymentItemModel)
  PaymentItems: PaymentItemModel[];
}
