import { ObjectBase, RecordNotFoundError } from '@tomei/general';
import { PaymentMethodTypeRepository } from './payment-method-type.repository';

export default class PaymentMethodType extends ObjectBase {
  MethodTypeId = 'New';
  MethodId = '';
  Name = '';
  AccountNo = '';
  ProcessingFeeRate = 0.0;
  ProcessingFeeAccountNo = '';

  private _DbTransaction: any;

  private static _RepositoryBase = new PaymentMethodTypeRepository();

  constructor(dbTransaction?: any, methodTypeId?: string) {
    super();
    if (dbTransaction) {
      this._DbTransaction = dbTransaction;
    }
    if (methodTypeId) {
      this.MethodTypeId = methodTypeId;
      PaymentMethodType._RepositoryBase
        .findOne({
          where: {
            MethodTypeId: this.MethodTypeId,
          },
          transaction: this._DbTransaction,
        })
        .then((paymentMethodTypeData) => {
          if (paymentMethodTypeData) {
            this.Name = paymentMethodTypeData.Name;
            this.AccountNo = paymentMethodTypeData.AccountNo;
            this.ProcessingFeeRate = paymentMethodTypeData.ProcessingFeeRate;
            this.ProcessingFeeAccountNo =
              paymentMethodTypeData.ProcessingFeeAccountNo;
          } else {
            throw new RecordNotFoundError('No record found.');
          }
        })
        .catch((err) => {
          console.log('Payment method type constructor err: ', err);
        });
    }
  }

  get ObjectId() {
    return this.MethodTypeId;
  }

  get ObjectName() {
    return this.Name;
  }

  get TableName() {
    return 'finance_PaymentMethodType';
  }
}
