import { ObjectBase, RecordNotFoundError } from '@tomei/general';
import { PaymentMethodRepository } from './payment-method.repository';
import PaymentMethodType from '../payment-method-type/payment-method-type';
import { PaymentMethodTypeRepository } from '../payment-method-type/payment-method-type.repository';
import { IPaymentMethodTypeAttr } from '../payment-method-type/interfaces/payment-method-type-attr.interface';
import FinanceCompany from '../finance-company/finance-company';

export default class PaymentMethod extends ObjectBase {
  MethodId = 'New';
  Name = '';
  CompanyId = '';

  private static _RepositoryBase = new PaymentMethodRepository();
  private static _PaymentMethodTypeRepository =
    new PaymentMethodTypeRepository();

  private _DbTransaction: any;
  private _PaymentMethodTypes = [];

  constructor(dbTransaction?: any, methodId?: string) {
    super();
    if (dbTransaction) {
      this._DbTransaction = dbTransaction;
    }
    if (methodId) {
      this.MethodId = methodId;
      PaymentMethod._RepositoryBase
        .findOne({
          where: {
            MethodId: methodId,
          },
          transaction: dbTransaction,
        })
        .then((methodData) => {
          if (methodData) {
            this.Name = methodData.Name;
          } else {
            throw new RecordNotFoundError('No record found.');
          }
        })
        .catch((err) => {
          console.log('Payment method constructor err: ', err);
        });
    }
  }

  get ObjectId() {
    return this.MethodId;
  }

  get ObjectName() {
    return this.Name;
  }

  get TableName() {
    return 'finance_PaymentMethod';
  }

  get PaymentMethodTypes(): Promise<PaymentMethodType[]> {
    return new Promise((resolve, reject) => {
      if (this.MethodId !== 'New') {
        PaymentMethod._PaymentMethodTypeRepository
          .findAll({
            where: {
              MethodId: this.MethodId,
            },
            transaction: this._DbTransaction,
          })
          .then((paymentMethodTypes) => {
            const paymentMethodTypeObjects = paymentMethodTypes.map(
              (paymentMethodTypeData) =>
                new PaymentMethodType(
                  this._DbTransaction,
                  paymentMethodTypeData.MethodTypeId,
                ),
            );
            return Promise.all(paymentMethodTypeObjects);
          })
          .then((paymentMethodTypeObjects) => {
            this._PaymentMethodTypes = paymentMethodTypeObjects;
            resolve(this._PaymentMethodTypes);
          })
          .catch((err) => {
            reject(err);
          });
      } else {
        resolve(this._PaymentMethodTypes);
      }
    });
  }

  /**
   * Create a new Payment Method Type which is automatically added to the Payment
   * Method Type list
   *
   * @see {@link FinanceCompany.LoadPaymentMethods} - for implementation
   *
   * @param {IPaymentMethodTypeAttr} paymentMethodTypePayload
   *
   * @returns {void}
   */
  async newPaymentMethodType(payload: IPaymentMethodTypeAttr): Promise<any> {
    try {
      const newPaymentMethodType =
        await PaymentMethod._PaymentMethodTypeRepository.create(payload);

      this._PaymentMethodTypes.push(newPaymentMethodType);
    } catch (err) {
      throw err;
    }
  }
}
